/*
	File:		CalendarSI.cpp

	Contains:	Implementation of Mac specific CalendarSI class.

	Owned by:	Nick Pilch

	Copyright:	 1994 - 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <7>	 5/21/95	NP		1249232: Gid rid of TokenInquiryProc.
									1250474: Change to CallGetMarkTokenProc.
									1248779: Change API of GetUserToken.
		 <6>	 4/28/95	eeh		1244258: remove IsValid and BaseRemoved
									override
		 <5>	 4/27/95	eeh		1230798: override IsValid and BaseRemoved
		 <4>	 1/27/95	NP		1213948: New classes for ODDesc et al.
		 <3>	 1/11/95	NP		#1188520-Pass real ODPart to parts.
		 <2>	  1/9/95	NP		1194880: CalendarSI name changes.
		 <1>	11/15/94	NP		first checked in
		 <7>	 9/23/94	NP		1188520-Pass the real ODPart descendent to
									the SemanticInterface extension rather than
									the ODPartWrapper.
		 <6>	  9/9/94	NP		1185851: implement scripting. 1184040: API
									changes.
		 <5>	  9/9/94	jpa		Removed call to parent in Release, at
									Nick's request. [1185656]
		 <4>	  7/8/94	NP		Initialize fHelper field.
		 <3>	  7/8/94	NP		Fixed bugs, implemented coercion handlers,
									fixed initialization.
		 <2>	  7/5/94	NP		Filled in CallEventHandler.
		 <1>	  7/1/94	NP		first checked in

	To Do:
*/


#ifndef _EXCEPT_
#include "Except.h"
#endif

#define VARIABLE_MACROS
#define CalendarSI_Class_Source
#include <CalendarSI.xih>

//------------------------------------------------------------------------------
// CalendarSI::somInit
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSIsomInit(CalendarSI *somSelf)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSIsomInit");

    CalendarSI_parents_somInit(somSelf);

	_fHelper = kODNULL;
}

//------------------------------------------------------------------------------
// CalendarSI::InitCPlusSemanticInterface
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSIInitCPlusSemanticInterface(CalendarSI *somSelf, Environment *ev,
		ODPart* base,
		SIHelperAbs* helper,
		ODSession* session)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSIInitCPlusSemanticInterface");

	if (somSelf->IsInitialized(ev))
		return;
	
	somSelf->InitSemanticInterface(ev, base, session);
	
	_fHelper = helper;
}

//------------------------------------------------------------------------------
// CalendarSI::somUninit
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSIsomUninit(CalendarSI *somSelf)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSIsomUninit");

    CalendarSI_parents_somUninit(somSelf);
}

//------------------------------------------------------------------------------
// CalendarSI::GetSIHelper
//------------------------------------------------------------------------------

SOM_Scope SIHelperAbs*  SOMLINK CalendarSIGetSIHelper(CalendarSI *somSelf, Environment *ev)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSIGetSIHelper");

    return _fHelper;
}

//------------------------------------------------------------------------------
// CalendarSI::CallEventHandler
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallEventHandler(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODAppleEvent* theODAppleEvent,
		ODAppleEvent* reply)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallEventHandler");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallEventHandler((ODPart*)somSelf->GetBase(ev),
												theODAppleEvent, reply);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallCoercionHandler
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallCoercionHandler(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODDesc* theODDesc,
		ODDescType toType,
		ODDesc* result)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallCoercionHandler");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallCoercionHandler((ODPart*)somSelf->GetBase(ev),
												theODDesc, toType, result);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallObjectAccessor
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallObjectAccessor(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODDescType desiredClass,
		ODOSLToken* container,
		ODDescType containerClass,
		ODDescType form,
		ODDesc* selectionData,
		ODOSLToken* value)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallObjectAccessor");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallObjectAccessor((ODPart*)somSelf->GetBase(ev),
												desiredClass, container,
												containerClass, form,
												selectionData, value);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallPredispatchProc
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallPredispatchProc(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODAppleEvent* theODAppleEvent,
		ODAppleEvent* reply)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallPredispatchProc");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallPredispatchProc((ODPart*)somSelf->GetBase(ev),
												theODAppleEvent, reply);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallCompareProc
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallCompareProc(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODDescType oper,
		ODOSLToken* obj1,
		ODOSLToken* obj2,
		ODBoolean* result)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallCompareProc");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallCompareProc((ODPart*)somSelf->GetBase(ev), oper,
												obj1, obj2, result);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallCountProc
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallCountProc(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODDescType desiredType,
		ODDescType containerClass,
		ODOSLToken* container,
		ODSLong* result)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallCountProc");

    OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallCountProc((ODPart*)somSelf->GetBase(ev),
											desiredType, containerClass,
											container, result);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallDisposeTokenProc
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallDisposeTokenProc(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODOSLToken* unneededToken)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallDisposeTokenProc");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallDisposeTokenProc((ODPart*)somSelf->GetBase(ev),
													unneededToken);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallGetErrDescProc
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallGetErrDescProc(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODDesc** errDesc)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallGetErrDescProc");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallGetErrDescProc((ODPart*)somSelf->GetBase(ev),
												errDesc);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallGetMarkTokenProc
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallGetMarkTokenProc(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODOSLToken* dContainerToken,
		ODDescType containerClass,
		ODOSLToken* result)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallGetMarkTokenProc");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallGetMarkTokenProc((ODPart*)somSelf->GetBase(ev),
													dContainerToken,
													containerClass, result);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallMarkProc
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallMarkProc(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODOSLToken* dToken,
		ODOSLToken* markToken,
		ODSLong index)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallMarkProc");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallMarkProc((ODPart*)somSelf->GetBase(ev), dToken,
											markToken, index);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// CalendarSI::CallAdjustMarksProc
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK CalendarSICallAdjustMarksProc(CalendarSI *somSelf, Environment *ev,
		ODPart* thePart,
		ODSLong newStart,
		ODSLong newStop,
		ODOSLToken* markToken)
{
    CalendarSIData *somThis = CalendarSIGetData(somSelf);
    CalendarSIMethodDebug("CalendarSI","CalendarSICallAdjustMarksProc");

	OSErr error;
	SOM_TRY
		somSelf->CheckValid(ev);
		error = _fHelper->CallAdjustMarksProc((ODPart*)somSelf->GetBase(ev),
												newStart, newStop, markToken);
		THROW_IF_ERROR(error);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

