//Parts of this file are
//Copyright  1995
//Manic Moose Consulting
//All rights reserved.
//You may use this code for reference purposed only
//moose@manicmoose.com


//----------------------------------------------------------------------------------------
// DragAndDrop protocol
//----------------------------------------------------------------------------------------

#ifndef _CALENDAR_
#include "CPCalendar.h"
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef SOM_ODDragItemIterator_xh
#include <DgItmIt.xh>
#endif

#ifndef SOM_ODDragAndDrop_xh
#include <DragDrp.xh>
#endif

#include "Day.h"

#if GENERATING68K
#pragma segment DragDrop
#endif

//----------------------------------------------------------------------------------------

ODBoolean CPCalendar::PrivValidDataOnSU(Environment* ev, ODStorageUnit *storageUnit)
{
	HFSFlavor *valueData;
	long valueSize;
	ODBoolean validDataWasFound = kODFalse;

	if (storageUnit->Exists(ev, kODPropContents, kODAppleTEXT, 0) || 
		 storageUnit->Exists(ev, kODPropContents, kCalendarKind, 0))
	{
		validDataWasFound = kODTrue;
	} else {
		if (storageUnit->Exists(ev, kODPropContents, kODApplehfs, 0))
		{
			storageUnit->Focus(ev, kODPropContents, kODPosUndefined, kODApplehfs, 0, kODPosUndefined);		
			valueSize = storageUnit->GetSize(ev);
			valueData = (HFSFlavor *)::NewPtr(valueSize);
			if (valueData)
			{
				StorageUnitGetValue(storageUnit, ev, valueSize, (ODValue)valueData);
				if (valueData->fileType == 'TEXT')
					validDataWasFound = kODTrue;
			}
			::DisposePtr((Ptr)valueData);
		}
	}
	return validDataWasFound;
}

//----------------------------------------------------------------------------------------

void CPCalendar::FulfillPromise(Environment* ev, ODStorageUnitView *promiseSUView)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_FulfillPromise
	EnteringMethod("\pCPCalendar::FulfillPromise");
	ODUnused(promiseSUView);
}

//----------------------------------------------------------------------------------------

void CPCalendar::DropCompleted(Environment* ev, ODPart* destPart, ODDropResult dropResult)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DropCompleted
	EnteringMethod("\pCPCalendar::DropCompleted");
	ODUnused(destPart);
	ODUnused(dropResult);
}

//----------------------------------------------------------------------------------------

ODDragResult CPCalendar::DragEnter(Environment* ev, ODDragItemIterator* dragInfo,
													 ODFacet* facet, ODPoint* where)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DragEnter
	EnteringMethod("\pCPCalendar::DragEnter");
	ODUnused(facet);
	ODUnused(where);

	ODStorageUnit *dragSU;
	
	fAcceptThisDrag = kODFalse;
	fActiveFrameHilitedForDrop = kODFalse;
	for (dragSU = dragInfo->First(ev); dragSU; dragSU = dragInfo->Next(ev))
	{
		fAcceptThisDrag = fAcceptThisDrag || this->PrivValidDataOnSU(ev,dragSU);
	}

	return fAcceptThisDrag;
}

//----------------------------------------------------------------------------------------

ODDragResult CPCalendar::DragWithin(Environment* ev, ODDragItemIterator* dragInfo,
													  ODFacet* facet, ODPoint* where)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DragWithin
	EnteringMethod("\pCPCalendar::DragWithin");
	ODUnused(dragInfo);
	ODUnused(where);

	if (fAcceptThisDrag)
	{
		CFocus myFocus(ev, facet);

		ODFrame* displayFrame	= facet->GetFrame(ev);
		ODShape* frameShape		= displayFrame->AcquireFrameShape(ev, kODNULL);
		ODULong attributes		= fSession->GetDragAndDrop(ev)->GetDragAttributes(ev);
		DragReference theDrag	= fSession->GetDragAndDrop(ev)->GetDragReference(ev);

		if (  ((attributes & kODDragIsInSourcePart) == kODFalse) )
		{
			ODFrame*	frame     = facet->GetFrame(ev);
			ODShape*	usedShape = frame->AcquireUsedShape(ev, kODNULL);
			RgnHandle	rgn       = usedShape->GetQDRegion(ev);
			Rect		rct     = (*rgn)->rgnBBox;
			Rect	hiliteFrame;
			Point	pt;
			GetDragMouse(theDrag, &pt, 0);
			GlobalToLocal(&pt);
			if (monthDrawer.CalcDayFrameFromPoint(rct, pt, hiliteFrame))
				{
					RgnHandle dragHiliteRgn =NewRgn();
					Day	*newLastDay = monthDrawer.GetDayInPoint(rct, pt);

					RectRgn(dragHiliteRgn, &hiliteFrame);
					if (fActiveFrameHilitedForDrop && (newLastDay != lastDayHilited))
						::HideDragHilite(theDrag);
					else
						::ShowDragHilite(theDrag, dragHiliteRgn, kODTrue);
					lastDayHilited = newLastDay;
					DisposeRgn(dragHiliteRgn);
					fActiveFrameHilitedForDrop = kODTrue;
				}
			usedShape->Release(ev);
		}
		frameShape->Release(ev);
	}

	return fAcceptThisDrag;
}

//----------------------------------------------------------------------------------------

void CPCalendar::DragLeave(Environment* ev, ODFacet* facet, ODPoint* where)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DragLeave
	EnteringMethod("\pCPCalendar::DragLeave");
	ODUnused(facet);
	ODUnused(where);

	ODULong attributes		= fSession->GetDragAndDrop(ev)->GetDragAttributes(ev);
	DragReference theDrag	= fSession->GetDragAndDrop(ev)->GetDragReference(ev);
	ODTransform* transform	= facet->AcquireWindowFrameTransform(ev, kODNULL);
	ODPoint windowWhere		= transform->TransformPoint(ev,where);
	transform->Release(ev);
	ODFrame* displayFrame	= facet->GetFrame(ev);
	
	if(fAcceptThisDrag)
	{
		CFocus myFocus(ev, facet);
		if (fActiveFrameHilitedForDrop) 
		{
			::HideDragHilite(theDrag);
			fActiveFrameHilitedForDrop = kODFalse;
		}
	}
}

//----------------------------------------------------------------------------------------

ODDropResult CPCalendar::Drop(Environment* ev, ODDragItemIterator* dropInfo,
												ODFacet* facet, ODPoint* where)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Drop
	EnteringMethod("\pCPCalendar::Drop");
	ODUnused(where);

	ODDropResult dropResult = kODDropFail;
	
	if (fActiveFrameHilitedForDrop)
	{
		short mouseDownModifiers;
		short mouseUpModifiers;
		Handle dropPicHandle = NULL;
		Boolean notDone = kODTrue;
	
		ODULong attributes = fSession->GetDragAndDrop(ev)->GetDragAttributes(ev);
		DragReference theDrag = fSession->GetDragAndDrop(ev)->GetDragReference(ev);
		::GetDragModifiers(theDrag, 0L, &mouseDownModifiers, &mouseUpModifiers);
		
		if (attributes & kODDropIsMove)
			dropResult = kODDropMove;
		else
			dropResult = kODDropCopy;
			
		// If option key is held down at mousedown, it is a force-copy.
		Boolean moveData = kODFalse;
		Boolean useLinkInfo = kODFalse;
		if (mouseDownModifiers & optionKey)
			moveData = kODFalse;
		// If cmd key is held down at mousedown, it is a Paste As.
		else if (mouseDownModifiers & cmdKey)
			useLinkInfo = kODTrue;
/*  When would the below code ever get called????
		// If option key is held down at mouseUp, it is a force-copy.
		else if (mouseUpModifiers & optionKey)
			moveData = kODFalse;
		// If cmd key is held down at mouseUp, it is a force-move.
		else if (mouseUpModifiers & cmdKey)
			useLinkInfo = kODTrue;
*/			
		// make new TextEdit record
		Rect theRect;
		::SetRect(&theRect, 0, 0, 0, 0);
		TEHandle tempTE			= TEStyleNew(&theRect, &theRect);
		StScrpHandle stylHandle = kODNULL;
		OSErr error				= noErr;
		short fileRefNum		= 0;
		Size textSize			= 0;
		ODPtr textData			= kODNULL;
		ODPtr fileData			= kODNULL;
	
		// iterate over all SU's and collect the styled text
		TRY
		for (ODStorageUnit* dropSU = dropInfo->First(ev); dropSU && notDone; dropSU = dropInfo->Next(ev))
		{
			// Try our native format first
			if (ODSUExistsThenFocus(ev, dropSU, kODPropContents, kCalendarKind))
			{
				textSize = dropSU->GetSize(ev);
				textData = ODNewPtr(textSize, kDefaultHeapID);
				StorageUnitGetValue(dropSU, ev, textSize, (ODValue)textData);
				
				notDone = kODFalse;
			}
			// Then try standard Apple Roman TEXT format
			else if (ODSUExistsThenFocus(ev, dropSU, kODPropContents, kODAppleTEXT))
			{
				textSize = dropSU->GetSize(ev);
				textData = ODNewPtr(textSize, kDefaultHeapID);
				StorageUnitGetValue(dropSU, ev, textSize, (ODValue)textData);
		
				if (ODSUExistsThenFocus(ev, dropSU, kODPropContents, kODApplestyl))
				{
					Size stylSize = dropSU->GetSize(ev);
					stylHandle = (StScrpHandle) ODNewHandle(stylSize);
					StorageUnitGetValue(dropSU, ev, stylSize, (ODValue) ODLockHandle((ODHandle) stylHandle));
					ODUnlockHandle((ODHandle) stylHandle);
				}
				notDone = kODFalse;
			}
			// And finally check to see if someone dropped a file, and just slurp in its entire data fork.
			else if (ODSUExistsThenFocus(ev, dropSU, kODPropContents, kODApplehfs))
			{
				dropSU->Focus(ev, kODPropContents, kODPosUndefined, kODApplehfs, 0, kODPosUndefined);		
				long fileSize = dropSU->GetSize(ev);		// really hfsflavor size
				fileData = ODNewPtr(fileSize, kDefaultHeapID);			// really hfsflavor data
				StorageUnitGetValue(dropSU, ev, fileSize, (ODValue)fileData);
				
				THROW_IF_ERROR(::FSpOpenDF((FSSpec*)&(((HFSFlavor*)fileData)->fileSpec), fsRdPerm, &fileRefNum));
				THROW_IF_ERROR(::GetEOF(fileRefNum, &textSize));
				if (textSize == 0)
					THROW(kODErrUndefined);
					
				textData = ODNewPtr(textSize, kDefaultHeapID);
				
				THROW_IF_ERROR(::SetFPos(fileRefNum,fsFromStart,0));
				THROW_IF_ERROR(::FSRead(fileRefNum,&textSize,textData));
				THROW_IF_ERROR(::FSClose(fileRefNum));
				notDone = kODFalse;
			}

			TESetSelect(32767, 32767, tempTE);
			TEStyleInsert(textData, textSize, stylHandle, tempTE);
	
			if (textData)
				ODDisposePtr(textData);
			if (stylHandle)
				ODDisposeHandle((ODHandle) stylHandle);
		}

		CATCH_ALL
		if (tempTE != kODNULL)
			::TEDispose(tempTE);
		if (fileData != kODNULL)
			ODDisposePtr(fileData);
		if (stylHandle != kODNULL)
			ODDisposeHandle((ODHandle) stylHandle);
		if (fileRefNum != 0)
			::FSClose(fileRefNum);
		dropResult = kODDropFail;
		ENDTRY

		if ((dropResult == kODDropCopy) || (dropResult == kODDropMove))
		{
			// OK, we got the styled text and we're now moving that into a Str255
			Size textSize = (**tempTE).teLength;
			ODPtr textData = ODNewPtr(textSize, kDefaultHeapID);
			if (! textData) 
			{
				DebugStr("\pNewPtr error");
				::TEDispose(tempTE);
				return kODDropFail ;
			}
			
			// In reality, you would preserve the style information
			if (textSize)
			{
				lastDayHilited->SetData(*(**tempTE).hText, textSize);
			}
			::TEDispose(tempTE);
			
			this->PrivSetDirty(ev);
			facet->Invalidate(ev, kODNULL, kODNULL);
		}
	}
	return dropResult;
}

