//----------------------------------------------------------------------------------------
// UI Events protocol
//----------------------------------------------------------------------------------------

#ifndef _CALENDAR_
#include "CPCalendar.h"
#endif

#ifndef SOM_ODClipboard_xh
#include <Clipbd.xh>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

//----------------------------------------------------------------------------------------

ODBoolean CPCalendar::HandleEvent(Environment* ev, ODEventData* event,
								   ODFrame* frame, ODFacet* facet,
								   ODEventInfo* eventInfo)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_HandleEvent
	EnteringMethod("\pCPCalendar::HandleEvent");
	ODUnused(eventInfo);

	ODBoolean	tWasHandled = kODFalse;

	switch (event->what)
	{
	case nullEvent:
		if (::IsDialogEvent((EventRecord *)event))
		{
			short itemHit;
			DialogPtr dialog;
			tWasHandled = ::DialogSelect((EventRecord *)event, &dialog, &itemHit);
			return tWasHandled;
		}
		tWasHandled = kODTrue;
		break;

	case mouseDown:
		tWasHandled = this->PrivHandleMouseDown(ev, event, frame, facet);
		break;

	case kODEvtMouseDownEmbedded:
	case mouseUp:
	case keyDown:
	case autoKey:
	case activateEvt:
		tWasHandled = kODTrue;
		break;
		
	case kODEvtMenu:
		tWasHandled = this->PrivHandleMenuEvent(ev, frame, event);
		break;
		
	default:
		break;
	}

	return tWasHandled;
}

//----------------------------------------------------------------------------------------

void CPCalendar::AdjustMenus(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_AdjustMenus
	EnteringMethod("\pCPCalendar::AdjustMenus");

	// fix up About command
	Str63 aboutText;
	{
		CUsingLibraryResources fil;
		::GetIndString(aboutText, kContentStringResID, kAboutTextID);
	}
	ODScriptCode script = ::GetScriptManagerVariable(smSysScript);
	ODLangCode lang = ::GetScriptVariable(script, smScriptLang);
	ODIText* odiText = CreateIText(script, lang, aboutText);
	fMenuBar->SetItemString(ev, kODCommandAbout, odiText);
	DisposeIText(odiText);
	
	fMenuBar->EnableCommand(ev, kODCommandViewAsWin, !frame->IsRoot(ev));

	// cut, copy, paste
	fMenuBar->EnableCommand(ev, kODCommandCut, kODTrue);
	fMenuBar->EnableCommand(ev, kODCommandCopy, kODTrue);
	fMenuBar->EnableCommand(ev, kODCommandClear, kODTrue);
	
	// fix Paste and Paste As
	ODBoolean okToPaste = kODFalse;
	ODBoolean okToPasteAs = kODFalse;
	ODBoolean okToShowLinkInfo = kODFalse;

	ODArbitrator* arbitrator = fSession->GetArbitrator(ev);
	ODTypeToken clipboardFocus = fSession->Tokenize(ev, kODClipboardFocus);
	ODFrame* clipboardOwner = arbitrator->AcquireFocusOwner(ev, clipboardFocus);
	
	if ((frame == clipboardOwner) ||
			  (arbitrator->RequestFocus(ev, clipboardFocus, frame)))
	{
		ODClipboard* clipboard = fSession->GetClipboard(ev);
		// Get the root storage unit for the clipboard
		ODStorageUnit* clipRootSU = clipboard->GetContentStorageUnit(ev);
		okToPaste = this->PrivValidDataOnSU(ev, clipRootSU);
		// must be a link in clipboard and not be a link to myself
		okToPasteAs = this->PrivValidLinkOnSU(ev, clipRootSU) &&
						(fClipboardUpdateID != clipboard->GetUpdateID(ev));
		arbitrator->RelinquishFocus(ev, clipboardFocus , frame);		
	}
	if (clipboardOwner)
		clipboardOwner->Release(ev);

	fMenuBar->EnableCommand(ev, kODCommandPaste, okToPaste);
	fMenuBar->EnableCommand(ev, kODCommandPasteAs, okToPasteAs);
	
	// fix up Part Info
	fMenuBar->EnableCommand(ev, kODCommandGetPartInfo, kODTrue);
	if ((fSrcLink || fDestLink) && TestKeyDown(kOptionKey))
	{
		fShowLinkInfo = kODTrue;
		fMenuBar->SetItemString(ev, kODCommandGetPartInfo, PStrToIntl("\pLink Info", kODNULL));
	}
	else
	{
		fShowLinkInfo = kODFalse;
		fMenuBar->SetItemString(ev, kODCommandGetPartInfo, PStrToIntl("\pPart Info", kODNULL));
	}
}

//----------------------------------------------------------------------------------------

ODSession* gSession;

ODBoolean CPCalendar::PrivHandleMenuEvent(Environment* ev, ODFrame* frame, ODEventData* event)
{
	EnteringMethod("\pCPCalendar::PrivHandleMenuEvent");
	ODUnused(frame);

	long	menuResult = event->message;
	short	menu = HiWord(menuResult);
	short	item = LoWord(menuResult);

	if (menu) {

		ODCommandID theMenuCommand = fMenuBar->GetCommand(ev, menu, item);
		switch (theMenuCommand) 
		{
		case kODCommandAbout:
		{
			CUsingLibraryResources fil;

			if (fSession->GetArbitrator(ev)->RequestFocus(ev, fModalFocus, frame))
			{
				::SetCursor(&ODQDGlobals.arrow);
				gSession = fSession;				// DialogFilter uses this!
				ModalFilterUPP modalFilter = NewModalFilterProc(MyDialogFilter);
				::Alert(kCalendarAboutBoxID, modalFilter);
				DisposeRoutineDescriptor(modalFilter);
				fSession->GetArbitrator(ev)->RelinquishFocus(ev, fModalFocus,frame);
			}
			else
				::SysBeep(1);
		}
		break;

		case kODCommandCut :
		case kODCommandCopy :
			{
			ODArbitrator* arbitrator = fSession->GetArbitrator(ev);
			ODTypeToken clipboardFocus = fSession->Tokenize(ev, kODClipboardFocus);
			ODFrame* clipboardOwner = arbitrator->AcquireFocusOwner(ev, clipboardFocus);
			if ((frame == clipboardOwner) ||
					  (arbitrator->RequestFocus(ev, clipboardFocus, frame)))
			{
				ODClipboard* clipboard = fSession->GetClipboard(ev);
				ODVolatile(clipboard);
				// Remove any existing data on the clipboard
				clipboard->Clear(ev);
				// Get the root storage unit for the clipboard
				ODStorageUnit* clipRootSU = clipboard->GetContentStorageUnit(ev);
				// Part specific code to write to the clipboard draft via 'clipRootSU'.
				
				ODDraftKey draftKey = 0;
				ODDraft* myDraft = fSelf->GetStorageUnit(ev)->GetDraft(ev);
				
				ODVolatile(myDraft);
				ODVolatile(draftKey);
				
				draftKey = myDraft->BeginClone(ev, clipRootSU->GetDraft(ev), kODNULL, kODCloneCopy);
				
				TRY
					this->PrivPrepareSU(ev, clipRootSU);
					this->PrivExternalizeToSU(ev, clipRootSU);
					if (theMenuCommand == kODCommandCopy)
						this->PrivExternalizeLinkToSU(ev, clipRootSU);
				CATCH_ALL
					myDraft->AbortClone(ev, draftKey);
					clipboard->Clear(ev);
					RERAISE;
				ENDTRY
				myDraft->EndClone(ev, draftKey);
				
				// memorize the change ID to detect cut/paste in myself
				fClipboardUpdateID = clipboard->GetUpdateID(ev);
				arbitrator->RelinquishFocus(ev, clipboardFocus , frame);
			}
			ODReleaseObject(ev, clipboardOwner);
			if (theMenuCommand == kODCommandCut)
			{
				fTextData[0] = 0;
				this->PrivInvalAllDisplayFrames(ev);
				this->PrivSetDirty(ev);
			}

			break;
			}

		case kODCommandClear:
			fTextData[0] = 0;
			this->PrivInvalAllDisplayFrames(ev);
			this->PrivSetDirty(ev);
			break;

		case kODCommandViewAsWin:
			this->Open(ev, frame);
			break;

		case kODCommandGetPartInfo:
			this->PrivGetPartInfo(ev, frame, event);
			break;

		case kODCommandPasteAs:
			this->PrivPasteAs(ev, frame, event);
			break;
			
		case kODCommandPaste :
			this->PrivPaste(ev, frame, event);
			break;
			
		default:
			return kODFalse;
		}
	}

	return kODTrue;
}

//------------------------------------------------------------------------------

void CPCalendar::PrivPasteAs(Environment* ev, ODFrame* frame, ODEventData* event)
{
	ODArbitrator* arbitrator = fSession->GetArbitrator(ev);
	ODTypeToken clipboardFocus = fSession->Tokenize(ev, kODClipboardFocus);
	
	ODFrame* clipboardOwner = arbitrator->AcquireFocusOwner(ev, clipboardFocus);
	if ((frame == clipboardOwner) ||
		(arbitrator->RequestFocus(ev, clipboardFocus, frame)))
	{
		ODClipboard* clipboard = fSession->GetClipboard(ev);
		ODFrameFacetIterator* facets = (ODFrameFacetIterator*) kODNULL;
		ODPasteAsResult pasteAsRslt;
		
		pasteAsRslt.pasteLinkSetting = kODTrue;
		TRY
		{
			ODStorageUnit* clipContentSU = clipboard->GetContentStorageUnit(ev);
			facets = frame->CreateFacetIterator(ev);
			if (clipboard->ShowPasteAsDialog(ev, 
						kODTrue,
						kODFalse,
						facets->First(ev),
						fSession->Tokenize(ev, kODViewAsFrame),
						&pasteAsRslt))
			{
				// Cannot paste a link if translation is required
				if (pasteAsRslt.translateKind != kODNULL)
					DebugStr("\pcannot translate !!!");
				// Get the root storage unit for the clipboard
				ODStorageUnit* clipRootSU = clipboard->GetContentStorageUnit(ev);
				this->PrivInternalizeFromSU(ev, clipRootSU);
				this->PrivInternalizeLinkFromSU(ev, clipRootSU, &pasteAsRslt);
				arbitrator->RelinquishFocus(ev, clipboardFocus , frame);		
				this->PrivSetDirty(ev);
			}
		}
		CATCH_ALL
			somPrintf("DoPasteAs caught error %d\n", ErrorCode());
		ENDTRY
		delete facets;
		if (pasteAsRslt.pasteLinkSetting == kODFalse)
			this->PrivPaste(ev, frame, event);
	}
	ODReleaseObject(ev, clipboardOwner);
}

//------------------------------------------------------------------------------

void CPCalendar::PrivPaste(Environment* ev, ODFrame* frame, ODEventData* event)
{
	ODArbitrator* arbitrator = fSession->GetArbitrator(ev);
	ODTypeToken clipboardFocus = fSession->Tokenize(ev, kODClipboardFocus);
	
	ODFrame* clipboardOwner = arbitrator->AcquireFocusOwner(ev, clipboardFocus);
	if ((frame == clipboardOwner) ||
			  (arbitrator->RequestFocus(ev, clipboardFocus, frame)))
	{
		ODClipboard* clipboard = fSession->GetClipboard(ev);
		// Get the root storage unit for the clipboard
		ODStorageUnit* clipRootSU = clipboard->GetContentStorageUnit(ev);
		Boolean gotSomething = this->PrivInternalizeFromSU(ev, clipRootSU, frame);
		if (gotSomething)
			this->PrivRemoveDestLink(ev);
		arbitrator->RelinquishFocus(ev, clipboardFocus, frame);		
		this->PrivSetDirty(ev);
	}
	ODReleaseObject(ev, clipboardOwner);
}

//------------------------------------------------------------------------------

ODBoolean CPCalendar::PrivHandleMouseDown(Environment* ev, ODEventData* event,
										 ODFrame* frame, ODFacet* facet)
{
	EnteringMethod("\pCPCalendar::PrivHandleMouseDown");

	ODBoolean handled = kODFalse;

	if (facet == kODNULL)		// Did we get a click outside a modal dialog? 
	{
		::SysBeep(1);
		return kODTrue;			// Return that click was handled.
	}

	// Activate the window if it is not active
	if (! facet->GetWindow(ev)->IsActive(ev))
	{
		// select the window
		facet->GetWindow(ev)->Select(ev);

		// typically you continue handling the event, if not, just return kODTrue
		handled = kODTrue;
		
		// return kODTrue; // user needs to click twice to get something done
	}

	// dialog events first
	if (::IsDialogEvent((EventRecord *)event))
	{
		short		itemHit;
		DialogPtr	dialog;

		handled = ::DialogSelect((EventRecord *)event, &dialog, &itemHit);
		// ODWindow* window = fSession->GetWindowState(ev)->AcquireODWindow(ev, dialog);
		return handled;		// In preparation for more "if" cases to be added below.
	}

	// get the mouse in local coordinates
	ODWindow* 	odwnd = frame->AcquireWindow(ev);
	ODPoint 	odPoint;
	WindowPtr	wnd   = (WindowPtr)odwnd->GetPlatformWindow(ev);
	GetWindowPoint(odwnd, ev, event->where, &odPoint);
	odwnd->Release(ev);

	// test for click in grow box, if appropriate
	if (frame->IsRoot(ev))
	{

		Rect r = wnd->portRect;
		r.left = r.right  - 15;
		r.top  = r.bottom - 15;
		if (::PtInRect(odPoint.AsQDPoint(), &r)) {
			::SetRect(&r, 200, 200, 20000, 20000);
			long newWindowSize = ::GrowWindow(wnd, event->where, &r);
			::SizeWindow(wnd, LoWord(newWindowSize), HiWord(newWindowSize), true);

			ODWindow* 	odwnd = frame->AcquireWindow(ev);
			odwnd->AdjustWindowShape(ev);
			odwnd->Release(ev);

			return kODTrue;
		}
	}

	if (fSession->GetArbitrator(ev)->RequestFocusSet(ev, fFocusSet,frame))
	{
		// Try to grab all of our foci
		this->FocusAcquired(ev, fSelectionFocus, frame);
		this->FocusAcquired(ev, fMenuFocus, frame);
		this->FocusAcquired(ev, fKeyFocus, frame);

	} else {
		// We couldn't get focus we requested, so bail.
		// Alternatively, parts of functionality can be disabled.
		return kODFalse;
	}

	// Handle clicks in used shape
	ODTransform* transform = facet->AcquireWindowContentTransform(ev, kODNULL);
	odPoint = transform->InvertPoint(ev, &odPoint);
	transform->Release(ev);
	
	ODShape*  usedShape = this->PrivAcquireUsedShape(ev, frame);
	ODBoolean hit       = usedShape->ContainsPoint(ev, &odPoint);
	usedShape->Release(ev);

	if (hit)
		handled = this->PrivUsedShapeHandleMouseDown(ev, event, frame, facet);

	return handled;
}

//------------------------------------------------------------------------------

ODBoolean CPCalendar::PrivUsedShapeHandleMouseDown(Environment* ev,
		ODEventData* event, ODFrame* frame, ODFacet* facet)
{
	EnteringMethod("\pCPCalendar::PrivUsedShapeHandleMouseDown");

	// Get the window and platform window (i.e. WindowPtr)
	ODWindow* odwnd = frame->AcquireWindow(ev);
	ODPlatformWindow facetPort = odwnd->GetPlatformWindow(ev);

	// Get the mouse in window-local and facet-local coordinates
	ODPoint windowPoint;
	GetWindowPoint(odwnd, ev, event->where, &windowPoint);
	odwnd->Release(ev);
	ODTransform* transform = facet->AcquireWindowContentTransform(ev, kODNULL);
	ODPoint facetPoint = transform->InvertPoint(ev, &windowPoint);
	transform->Release(ev);

	// Handle the click in whatever way is appropriate
	{
		CFocus  foc(ev, facet);
		Point	macPoint = facetPoint.AsQDPoint();
		// You now have the click point in fact coordinate space, so do something.
	}

	return kODTrue;		// We handled the click.
}