//----------------------------------------------------------------------------------------
// Extensions protocol
//----------------------------------------------------------------------------------------

#ifndef _CALENDAR_
#include "CPCalendar.h"
#endif

#ifndef SOM_Module_OpenDoc_StandardExtensions_defined
#include <StdExts.xh>
#endif

#ifndef SOM_CalendarSI_xh
#include <CalendarSI.xh>
#endif

#ifndef SOM_ODNameResolver_xh
#include <NamRslvr.xh>
#endif

#ifndef SOM_ODAppleEvent_xh
#include <ODAplEvt.xh>
#endif

#ifndef SOM_ODOSLToken_xh
#include <ODOSLTkn.xh>
#endif

#ifndef _SIHELPER_
#include <SIHelper.h>
#endif

#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif

#ifndef _ODDESUTL_
#include <ODDesUtl.h>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef _LIMITS
#include <limits.h>
#endif

void GetDirectParam(ODSession* session, AppleEvent* message,
		AEDesc* evtDp);

void UpdateUserToken(Environment* ev, ODNameResolver* resolver,
						ODOSLToken* odToken, AEDesc* desc);

#define GETBYTESOFHANDLE(h,typ,skipbytes)								\
	*((typ*)((*(h))+(skipbytes)))										\

#define FIRSTBYTESFROMHANDLE(h, typ)									\
		GETBYTESOFHANDLE(h,typ,0)										\


//----------------------------------------------------------------------------------------

ODBoolean CPCalendar::HasExtension(Environment* ev, ODType extensionName)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_HasExtension
	EnteringMethod("\pCPCalendar::HasExtension");
	ODUnused(extensionName);

	if (!ODISOStrCompare((ODISOStr)extensionName, kODExtSemanticInterface))
		return kODTrue;
	else // check for other extension API's you support here.
		return kODFalse;
}

//----------------------------------------------------------------------------------------

ODExtension* CPCalendar::AcquireExtension(Environment* ev, ODType extensionName)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_AcquireExtension
	EnteringMethod("\pCPCalendar::AcquireExtension");
	ODUnused(extensionName);

		// If we are called with an extension type we don't support,
		// we should be returning an error here.
	if (!ODISOStrCompare((ODISOStr)extensionName, kODExtSemanticInterface))
	{
		// We lazily construct our Semantic Interface extension here. Then we
		// hold onto it until we are released. Note that we must bump the
		// refcount of the extension anytime we are called.
		if (!fSemanticInterface)
			this->PrivConstructSemanticInterface(ev);
		else
			fSemanticInterface->Acquire(ev);
			
		return fSemanticInterface;
	}
	else // check for other extension API's you support here.
	{
		ev->_major = kODErrUnsupportedExtension;
		return kODNULL;
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::ReleaseExtension(Environment* ev, ODExtension* extension)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_ReleaseExtension
	EnteringMethod("\pCPCalendar::ReleaseExtension");
	ODUnused(extension);

	// If this method is called, the refCount of your extension has fallen to
	// 0 and you should delete it.
	if (extension == fSemanticInterface)
		ODDeleteObject(fSemanticInterface);
	else
		ev->_major = kODErrInvalidExtension;
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivConstructSemanticInterface(Environment* ev)
{
	// The following lines must be used in strict order in order to initalize
	// The CalendarSI and SIHelper classes correctly.
	fSemanticInterface = new CalendarSI;
	SIHelper* semItfcHelper = new SIHelper;
	semItfcHelper->InitSIHelper(fSession, fSemanticInterface);

	// The Extension interface requires that a pointer to the real part, not the
	// part wrapper, be passed to InitExtension. Therefore, we get that
	// reference here.
	ODPart*	realPart = fSelf->GetRealPart(ev);
	fSemanticInterface->InitCPlusSemanticInterface(ev, realPart, semItfcHelper, fSession);
	fSelf->ReleaseRealPart(ev);
	
	SIHelper* helper = (SIHelper*)fSemanticInterface->GetSIHelper(ev);

	// We install one object accessor in order to get a token for the contents
	// property and two event handlers to handle get data and set data.

	ODObjectAccessorUPP theAccessorUPP ;

	theAccessorUPP = NewODObjectAccessorProc( GetPropertyFromNULL ) ;
	helper->InstallObjectAccessor(cProperty, typeNull, theAccessorUPP,
										(ODSLong)this);

	ODEventHandlerUPP theEventHandlerUPP;

	theEventHandlerUPP = NewODEventHandlerProc( HandleSetData ) ;
	helper->InstallEventHandler(kAECoreSuite, kAESetData,
												theEventHandlerUPP,
												(ODSLong)this);

	theEventHandlerUPP = NewODEventHandlerProc( HandleGetData ) ;
	helper->InstallEventHandler(kAECoreSuite, kAEGetData,
												theEventHandlerUPP,
												(ODSLong)this);

	// Here's how you would install a compare proc, count proc, and dispose
	// token proc if you want to support whose clause resolution.
/*			
	ODCompareUPP compUPP = NewODCompareProc( DrawCompare ) ;
	helper->InstallCompareProc(compUPP, (ODSLong)fSession);

	ODCountUPP countUPP = NewODCountProc( DrawCount ) ;
	helper->InstallCountProc(countUPP, (ODSLong)fSession);
*/

	// Here's where you would install a dispose token proc if your tokens
	// contain pointers to other data structures you need to dispose
	// or cannot otherwise be cleanly disposed by AEDisposeDesc
/*
	ODDisposeTokenUPP disposeTokenUPP
			= NewODDisposeTokenProc( DrawDisposeToken ) ;
	helper->InstallDisposeTokenProc( disposeTokenUPP, (ODSLong)fSession);
*/

}

//------------------------------------------------------------------------------

// Just a convenience definition.
#define NULL_DESCRIPTOR_DEFINITION {typeNull, NULL}

//------------------------------------------------------------------------------

pascal ODError CPCalendar::GetPropertyFromNULL(
						ODPart*				part,
						DescType			desiredClass,
						ODOSLToken*			container,
						DescType			containerClass,
						DescType			form,
						ODDesc*				selectionData,
						ODOSLToken*			value,
						ODSLong				refCon)
{
	ODUnused(desiredClass);
	ODUnused(container);
	ODUnused(containerClass);

	Environment*		ev 			= somGetGlobalEnvironment();
	CPCalendar*		self		= (CPCalendar*)refCon;
	ODSession*			session 	= self->fSession;
	ODNameResolver*		resolver	= session->GetNameResolver(ev);
	DescType			propID;
	
	ODError				error = noErr;

	AEDesc				tokenDesc = NULL_DESCRIPTOR_DEFINITION;
	AEDesc				realData = NULL_DESCRIPTOR_DEFINITION;
	
	ODVolatile(tokenDesc);
	ODVolatile(realData);

	TRY
		// We throw errAEEventNotHandled so that OpenDoc's default Semantic
		// Interface will try to handle this access even if we couldnt't
		if (form != formPropertyID)
			THROW(errAEEventNotHandled);
	
		THROW_IF_ERROR( ODDescToAEDesc( selectionData, &realData ) );

		propID = FIRSTBYTESFROMHANDLE(realData.dataHandle, DescType);
	
		switch( propID )
		{
			case pContents:
				// Create a descriptor that represents the object that we are
				// being asked for. For this object, which is the contents
				// property, we choose to set the descriptorType to cProperty
				// and to put the id of the property into the first four bytes
				// of the handle of the descriptor. We could have chosen a
				// different representation if we wanted to. It is also
				// common to instantiate a C++ object and place a reference to
				// it in the dataHandle of the token.
				THROW_IF_ERROR(AECreateDesc(cProperty, &propID, sizeof(propID),
												&tokenDesc));
	
				// if your part's contents vary from display frame to display
				// frame then use the below code to determine which frame this
				// object access applies to.
	/*
				ODPart* contextPart;
				ODFrame* contextFrame;
				resolver->GetContextFromToken( ev, container, &contextPart,
						&contextFrame);
	*/
	
				// This utility routines gets the user token from the token and
				// updates it with the data passed in. See SEUtils.cpp.
				UpdateUserToken(ev, resolver, value, &tokenDesc);
				break;
				
			default:
				THROW(errAEEventNotHandled);		// let the default try
				break;
		}
	CATCH_ALL
		error = ErrorCode();
	ENDTRY

	AEDisposeDesc(&tokenDesc);
	AEDisposeDesc(&realData);

	return error;
}

//----------------------------------------------------------------------------------------

pascal ODError CPCalendar::HandleSetData(
						ODPart*			part,
						ODAppleEvent*	message,
						ODAppleEvent*	reply,
						ODSLong			handlerRefcon)
{
	ODUnused(part);
	ODUnused(reply);

	// W take advantage of Apple events disposal routine, AEDisposeDesc.
	// AEDisposeDesc will just return if passed a null descriptor, like the
	// ones we're creating just below by initializing them with
	// "NULL_DESCRIPTOR_DEFINITION" ({typeNull, NULL}).
	// At the bottom of the function we call AEDisposeDesc on all the
	// descriptors we may have created, knowing that the ones that have been
	// allocated will get disposed and the ones that were never allocated will
	// not cause AEDisposeDesc any problems.

	AppleEvent	realMessage = NULL_DESCRIPTOR_DEFINITION;
	AEDesc		evtDp = NULL_DESCRIPTOR_DEFINITION;
	AEDesc		theData = NULL_DESCRIPTOR_DEFINITION;
	AEDesc		theToken = NULL_DESCRIPTOR_DEFINITION;
	AEDesc		massagedData = NULL_DESCRIPTOR_DEFINITION;
	ODOSLToken*	tokenODDesc = kODNULL;

	// Since these variables are examined in or after the CATCH block, we must
	// declare them volatile.  Note that it isn't strictly necessary for all of
	// these to be declared VOLATILE, since the addresses of some are taken via
	// the & operator and others are too big to fit in a register.  But it doesn't
	// hurt to do this, and it makes us feel safer.

	ODVolatile(realMessage);
	ODVolatile(evtDp);
	ODVolatile(theData);
	ODVolatile(theToken);
	ODVolatile(massagedData);
	ODVolatile(tokenODDesc);

	ODError	error = noErr;

	CPCalendar*	self		= (CPCalendar*)handlerRefcon;
	ODSession*		session 	= self->fSession;
	Environment*	ev = somGetGlobalEnvironment();
	ODNameResolver* resolver =
			session->GetNameResolver(ev);
	
	TRY
		// Get an AEDesc version of our Apple event.
		THROW_IF_ERROR( ODDescToAEDesc(message, &realMessage) );

		// Get the data that will using for "setting".
		THROW_IF_ERROR(
			AEGetKeyDesc(&realMessage, keyAEData, typeWildCard, &theData));

		// Get the direct parameter of the event. This will be our token.
		AEDesc evtDp = NULL_DESCRIPTOR_DEFINITION;
		GetDirectParam(session, &realMessage, &evtDp);

		// We need to create an ODDesc version of this so that we can call the
		// OpenDoc routines IsODToken and GetUserToken with it.
		tokenODDesc = new ODOSLToken();
		tokenODDesc->InitODOSLToken(ev);
		THROW_IF_ERROR( AEDescToODDesc(&evtDp, tokenODDesc ) );

		// We might not actually get a token in the direct parameter. We
		// need to make sure it's an ODToken.
		if ( !resolver->IsODToken( ev, tokenODDesc ) )
			THROW( errAENoSuchObject );

		// Get the "user" token. This is the one that we have actually
		// modified in our object accessor routine earlier.
		ODDesc* myTokenODDesc;
		myTokenODDesc = resolver->GetUserToken(ev, tokenODDesc);

		// Get an AEDesc version of same. Note that we could have extracted the
		// data using ODDesc::GetRawData as well.
		THROW_IF_ERROR(ODDescToAEDesc(myTokenODDesc, &theToken));

		switch (theToken.descriptorType)
		{
			case typeProperty:
				// We know that if the descriptorType is typeProperty, we
				// earlier placed a property ID DescType in the first four
				// bytes of the dataHandle.
				DescType propID =
					FIRSTBYTESFROMHANDLE(theToken.dataHandle, DescType);
				if (propID != pContents)
					THROW(errAEEventNotHandled);
				else
				{
					Size	textLength;

					// Make sure the data is in a form we can work with. Note
					// that the local variable massagedData contains the data
					// after the following few lines of code are executed.
					// That's why we can dispose theData here. 
					if (theData.descriptorType != typeChar)
					{
						THROW_IF_ERROR(AECoerceDesc(&theData, typeChar,
													&massagedData));
						AEDisposeDesc(&theData);
					}
					else
					{
						massagedData = theData;
						theData.dataHandle = NULL;
						theData.descriptorType = typeNull;
					}

					// Get the data out of the dataHandle and change our
					// private data.
					textLength = GetHandleSize(massagedData.dataHandle);
					THROW_IF_ERROR(MemError());
					if (textLength > UCHAR_MAX)
						THROW(errAEEventNotHandled);

					self->fTextData[0] = textLength;
					ODBlockMove(*(massagedData.dataHandle),
								&(self->fTextData[1]), textLength);

					// Invalidate all of the frames since the data is
					// displayed in all of them. If we were concerned with
					// invalidating only a particular frame, we could have
					// gotten that info with GetContextFromToken. We're done!
					self->PrivInvalAllDisplayFrames(ev);
					self->PrivSetDirty(ev);
				}
				break;
			
			default:
				THROW(errAEEventNotHandled);
		}
	CATCH_ALL
		error = ErrorCode();
	ENDTRY

	// Clean up
	AEDisposeDesc(&realMessage);
	AEDisposeDesc(&evtDp);
	AEDisposeDesc(&theData);
	AEDisposeDesc(&theToken);
	AEDisposeDesc(&massagedData);
	ODDeleteObject( tokenODDesc );

	return error;
}


//------------------------------------------------------------------------------

pascal ODError CPCalendar::HandleGetData(
						ODPart*			part,
						ODAppleEvent*	message,
						ODAppleEvent*	reply,
						ODSLong			handlerRefcon)
{
	ODUnused(part);
	ODUnused(handlerRefcon);

	DescType		reqType;
	Size			theSize;

	ODError			error;

	Environment*	ev = somGetGlobalEnvironment();
	CPCalendar*	self		= (CPCalendar*)handlerRefcon;
	ODSession*		session 	= self->fSession;
	ODNameResolver* resolver = session->GetNameResolver(ev);

	AEDesc			evtDp = NULL_DESCRIPTOR_DEFINITION;
	AEDesc			objectData = NULL_DESCRIPTOR_DEFINITION;
	AEDesc			token = NULL_DESCRIPTOR_DEFINITION;
	AppleEvent		realMessage = NULL_DESCRIPTOR_DEFINITION;
	AppleEvent		realReply = NULL_DESCRIPTOR_DEFINITION;

	ODOSLToken* tokenODDesc = kODNULL;

	ODVolatile(realMessage);
	ODVolatile(evtDp);
	ODVolatile(objectData);
	ODVolatile(token);
	ODVolatile(realReply);
	ODVolatile(tokenODDesc);

	TRY
		// Get AEDesc versions of the event and the reply
		THROW_IF_ERROR( ODDescToAEDesc(message, &realMessage));
		THROW_IF_ERROR( ODDescToAEDesc(reply, &realReply));

		// Get the direct parameter which will contain our token. We must also
		// create an ODDesc version so that we can call the OpenDoc methods
		// IsODToken and GetUserToken.
		GetDirectParam(session, &realMessage, &evtDp);
		tokenODDesc = new ODOSLToken();
		tokenODDesc->InitODOSLToken(ev);
		THROW_IF_ERROR( AEDescToODDesc(&evtDp, tokenODDesc ) );

		// We might not actually get a token in the direct parameter. We
		// need to make sure it's an ODToken.
		if ( !resolver->IsODToken( ev, tokenODDesc ) )
			THROW( errAENoSuchObject );

		// Get the "user" token which contains the token that we set up earlier.
		ODDesc* myTokenODDesc;
		myTokenODDesc = resolver->GetUserToken(ev, tokenODDesc);
		THROW_IF_ERROR( ODDescToAEDesc(myTokenODDesc, &token ) );
	
		// Next, get the requested return type, if it exists.
		error = AEGetParamPtr(&realMessage, keyAERequestedType, typeType,
						&reqType, (Ptr)&reqType, sizeof(reqType), &theSize);
		// Not an error if return type is not found.
		if (error == errAEDescNotFound)
		{
			error = noErr;
			reqType = typeWildCard;
		}
		else
			THROW_IF_ERROR(error);
	
		switch (token.descriptorType)
		{
			case typeProperty:
				// Create a descriptor containing our data. We try to coerce
				// it if we were asked for a type that's different from the
				// one we would ordinarily create.
				THROW_IF_ERROR(AECreateDesc(typeChar, &(self->fTextData[1]),
											self->fTextData[0], &objectData));
				if (reqType != typeChar && reqType != typeWildCard)
				{
					AEDesc	newData = NULL_DESCRIPTOR_DEFINITION;
					THROW_IF_ERROR(AECoerceDesc(&objectData, reqType, &newData));
					AEDisposeDesc(&objectData);
					objectData = newData;
				}
				break;
			default:
				THROW(errAEEventNotHandled);
			
		}
		
		// Place the result data into the reply
		THROW_IF_ERROR(AEPutParamDesc( &realReply, keyAEResult, &objectData ));

		// Update the ODDesc version of the reply. The AEDesc version is just
		// a local copy that we use so that we can use all the AE manager
		// routines on it.
		THROW_IF_ERROR( AEDescToODDesc( &realReply, reply ) );
	CATCH_ALL
		error = ErrorCode();
	ENDTRY
	
	AEDisposeDesc(&evtDp);
	AEDisposeDesc(&objectData);
	AEDisposeDesc(&realReply);
	AEDisposeDesc(&realMessage);
	AEDisposeDesc(&token);
	ODDeleteObject( tokenODDesc );

	return error;
}

void GetDirectParam(ODSession* session, AppleEvent* message, AEDesc* evtDp)
{
	AEDesc			localDP = NULL_DESCRIPTOR_DEFINITION;
	Environment*	ev = somGetGlobalEnvironment();
	ODNameResolver* resolver = session->GetNameResolver(ev);

	// Get the direct parameter
	THROW_IF_ERROR(AEGetParamDesc(message, keyDirectObject, typeWildCard,
									&localDP));

	// Make an ODDesc from it.
	ODOSLToken* tmpWrapper = new ODOSLToken();
	tmpWrapper->InitODOSLToken(ev);
	THROW_IF_ERROR( AEDescToODDesc(&localDP, tmpWrapper ) );

	// Is it an OD token?
	ODBoolean isToken = resolver->IsODToken(ev, tmpWrapper);
	ODDeleteObject( tmpWrapper );

	if ( isToken )
		*evtDp = localDP;
	else
	{
		AEDisposeDesc(&localDP);
		THROW( errAEEventNotHandled );
	}
}

//------------------------------------------------------------------------------

void UpdateUserToken(Environment* ev, ODNameResolver* resolver,
						ODOSLToken* odToken, AEDesc* desc)
{
	ODDesc*	userODToken = resolver->GetUserToken(ev, odToken);
	THROW_IF_ERROR(AEDescToODDesc(desc, userODToken));
	THROW_IF_ERROR(AEDisposeDesc(desc));
}

