//----------------------------------------------------------------------------------------
// Frame protocol
//----------------------------------------------------------------------------------------

#ifndef _CALENDAR_
#include "CPCalendar.h"
#endif

//----------------------------------------------------------------------------------------

void CPCalendar::DisplayFrameAdded(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DisplayFrameAdded
	EnteringMethod("\pCPCalendar::AddDisplayFrame");

	if (frame->IsRoot(ev))
	{
		ODShape* shape = this->PrivAcquireFrameShape(ev, frame);
		ODShape* newShape = frame->RequestFrameShape(ev, shape, kODNULL);
			// First, PrivAcquireFrameShape is called to calculate the frame
			// shape we want.  However, we might not get it, due to the
			// needs/constraints of the containing part.  Therefore, after
			// calculating what we want, we see if we can get it by calling
			// RequestFrameShape.
			// RequestFrameShape either returns us what we passed it, or
			// returns something else because it thinks it knows best.
			// Either way, we then proceed with the returned result.
			// Note that this means that we have to be prepared to deal
			// with a frame shape other than we desire.
		shape->Release(ev);
		if (newShape)
			newShape->Release(ev);
	}
	
	fDisplayFrames->Add(frame, ev);
		// Add new frame to list of current display frames.
	
	frame->SetDroppable(ev, kODTrue);
		// We (almost) always want drag and drop in parts.
}

//----------------------------------------------------------------------------------------

void CPCalendar::DisplayFrameConnected(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DisplayFrameConnected
	EnteringMethod("\pCPCalendar::DisplayFrameConnected");

	// If your part editor supports 'kinds' which other part editors support
	// then your part editor might have been subsituted for one of those other part editors
	// and your assumptions about your display frames' shape and other attributes
	// may not be true.  You may need to check those attributes and adjust them appropriately.
	
	fDisplayFrames->Add(frame, ev);
		// Add frame to list of current display frames.
	
	frame->SetDroppable(ev, kODTrue);
		// We (almost) always want drag and drop in parts.
	
	ODShape* shape = this->PrivAcquireFrameShape(ev, frame);
	frame->ChangeUsedShape(ev, shape, kODNULL);
	shape->Release(ev);
	
	frame->Invalidate(ev, kODNULL, kODNULL);
}

//----------------------------------------------------------------------------------------

void CPCalendar::DisplayFrameRemoved(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DisplayFrameRemoved
	EnteringMethod("\pCPCalendar::DisplayFrameRemoved");

	fSession->GetArbitrator(ev)->RelinquishFocusSet(ev, fFocusSet, frame);

	fDisplayFrames->Remove(frame, ev);
		// This just removes the reference from our frame list.
}

//----------------------------------------------------------------------------------------

void CPCalendar::DisplayFrameClosed(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DisplayFrameClosed
	EnteringMethod("\pCPCalendar::DisplayFrameClosed");

	fSession->GetArbitrator(ev)->RelinquishFocusSet(ev, fFocusSet, frame);

	fDisplayFrames->Remove(frame, ev);
		// This just removes the reference from our frame list.
}

//----------------------------------------------------------------------------------------

void CPCalendar::FrameShapeChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_FrameShapeChanged
	EnteringMethod("\pCPCalendar::FrameShapeChanged");

	ODShape* shape = this->PrivAcquireFrameShape(ev, frame);
	ODShape* newShape = frame->RequestFrameShape(ev, shape, kODNULL);
	shape->Release(ev);
	if (newShape)
		newShape->Release(ev);
	this->PrivAdjustFacets(ev, frame);
	frame->Invalidate(ev, kODNULL, kODNULL);
	frame->InvalidateActiveBorder(ev);
}

//----------------------------------------------------------------------------------------

ODShape* CPCalendar::PrivAcquireFrameShape(Environment* ev, ODFrame* frame)
{
	// This is just to centralize the frame shape code.  Do it any way
	// is best for your part.

	ODShape* newFrameShape = (frame->IsRoot(ev)) ?
									frame->AcquireFrameShape(ev, kODNULL)	:
									this->PrivAcquireUsedShape(ev, frame)	;// T: it is being assigned, refbal does not yet understand the ?: expression.
	return newFrameShape;
}

//----------------------------------------------------------------------------------------

ODShape* CPCalendar::PrivAcquireUsedShape(Environment* ev, ODFrame* frame)
{
	// This is just to centralize the frame shape code.  Do it any way
	// is best for your part.
	
	ODShape* frameShape = frame->AcquireFrameShape(ev, kODNULL);
	RgnHandle	shapeRgn = frameShape->GetQDRegion(ev);
	Rect		rr;

	frameShape->Release(ev);

	ODTypeToken	theViewType = frame->GetViewType(ev);
		
	// handle Small Icon view
	if (theViewType == fSession->Tokenize(ev, kODViewAsSmallIcon))
	{
		::SetRect(&rr, 0, 0, kODSmallIconSize, kODSmallIconSize);
	}
	// handle Large Icon view
	else if (theViewType == fSession->Tokenize(ev, kODViewAsLargeIcon))
	{
		::SetRect(&rr, 0, 0, kODLargeIconSize, kODLargeIconSize);
	}
	// handle Thumbnail view
	else if (theViewType == fSession->Tokenize(ev, kODViewAsThumbnail))
	{
		::SetRect(&rr, 0, 0, kODThumbnailSize, kODThumbnailSize);
	}
	// handle Frame viewType
	else
	{
		rr = (*shapeRgn)->rgnBBox;
		short	hh = rr.bottom - rr.top;
		short	ww = rr.right - rr.left;
	
		::SetRect(&rr, 0, 0, ww, hh);
	}

	::RectRgn(shapeRgn = NewRgn(), &rr);

	ODShape* usedShape = frame->CreateShape(ev);
	usedShape->SetQDRegion(ev, shapeRgn);

	return usedShape;
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivAdjustFacets(Environment* ev, ODFrame* frame)
{
	// Given a frame, iterate through all of the facets and adjust them
	// according to the new frame.

	ODShape* usedShape  = this->PrivAcquireUsedShape(ev, frame);
	frame->ChangeUsedShape(ev, usedShape, kODNULL);
	usedShape->Release(ev);

	ODShape* frameShape = frame->AcquireFrameShape(ev, kODNULL);
	ODFrameFacetIterator* facets = frame->CreateFacetIterator(ev);
	for (ODFacet* facet = facets->First(ev);
			facets->IsNotComplete(ev);
			facet = facets->Next(ev) )
	{
		facet->ChangeActiveShape(ev, frameShape, kODNULL);
	}
	delete facets;
	frameShape->Release(ev);
}


//----------------------------------------------------------------------------------------

void CPCalendar::AttachSourceFrame(Environment* ev, ODFrame* frame, ODFrame* sourceFrame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_AttachSourceFrame
	EnteringMethod("\pCPCalendar::AttachSourceFrame");
	ODUnused(frame);
	ODUnused(sourceFrame);
}

//----------------------------------------------------------------------------------------

void CPCalendar::ViewTypeChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_ViewTypeChanged
	EnteringMethod("\pCPCalendar::ViewTypeChanged");

	// Invalidate the entire frame
	frame->Invalidate(ev, kODNULL, kODNULL);
	frame->InvalidateActiveBorder(ev);
}

//----------------------------------------------------------------------------------------

void CPCalendar::PresentationChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_PresentationChanged
	EnteringMethod("\pCPCalendar::PresentationChanged");
	ODUnused(frame);
}

//----------------------------------------------------------------------------------------

void CPCalendar::SequenceChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_SequenceChanged
	EnteringMethod("\pCPCalendar::SequenceChanged");
	ODUnused(frame);
}

//----------------------------------------------------------------------------------------

void CPCalendar::WritePartInfo(Environment* ev, ODInfoType partInfo,
							   ODStorageUnitView* storageUnitView)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_WritePartInfo
	EnteringMethod("\pCPCalendar::WritePartInfo");
	ODUnused(partInfo);
	ODUnused(storageUnitView);
}

//----------------------------------------------------------------------------------------

void CPCalendar::ClonePartInfo(Environment *ev, ODDraftKey key,
								ODInfoType partInfo,
								ODStorageUnitView* storageUnitView,
								ODFrame* scopeFrame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_ClonePartInfo
	EnteringMethod("\pCPCalendar::ClonePartInfo");
	ODUnused(key);
	ODUnused(partInfo);
	ODUnused(storageUnitView);
	ODUnused(scopeFrame);
}

//----------------------------------------------------------------------------------------

ODInfoType CPCalendar::ReadPartInfo(Environment* ev, ODFrame* frame, ODStorageUnitView* storageUnitView)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_ReadPartInfo
	EnteringMethod("\pCPCalendar::ReadPartInfo");
	ODUnused(frame);
	ODUnused(storageUnitView);
	return kODNULL;
}

//----------------------------------------------------------------------------------------

ODFrame* CPCalendar::RequestEmbeddedFrame(Environment* ev, ODFrame* containingFrame,
										   ODFrame* baseFrame,
										   ODShape* frameShape,
										   ODPart* embedPart,
										   ODTypeToken viewType,
										   ODTypeToken presentation,
										   ODBoolean isOverlaid)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_RequestEmbeddedFrame
	EnteringMethod("\pCPCalendar::RequestEmbeddedFrame");
	ODUnused(containingFrame);
	ODUnused(baseFrame);
	ODUnused(frameShape);
	ODUnused(embedPart);
	ODUnused(viewType);
	ODUnused(presentation);
	ODUnused(isOverlaid);
	DebugStr("\pCPCalendar::RequestEmbeddedFrame -- shouldn't be here.");
	return kODNULL;
}

//----------------------------------------------------------------------------------------

void CPCalendar::RemoveEmbeddedFrame(Environment* ev, ODFrame* embeddedFrame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_RemoveEmbeddedFrame
	EnteringMethod("\pCPCalendar::RemoveEmbeddedFrame");
	ODUnused(embeddedFrame);
	DebugStr("\pCPCalendar::RemoveEmbeddedFrame -- shouldn't be here.");
}

//----------------------------------------------------------------------------------------

ODShape* CPCalendar::RequestFrameShape(Environment* ev, ODFrame* embeddedFrame, ODShape* frameShape)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_RequestFrameShape
	EnteringMethod("\pCPCalendar::RequestFrameShape");
	ODUnused(embeddedFrame);
	ODUnused(frameShape);
	DebugStr("\pCPCalendar::RequestFrameShape -- shouldn't be here.");
	return kODNULL;
}

//----------------------------------------------------------------------------------------

void CPCalendar::UsedShapeChanged(Environment* ev, ODFrame* embeddedFrame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_UsedShapeChanged
	EnteringMethod("\pCPCalendar::UsedShapeChanged");
	ODUnused(embeddedFrame);
}

//----------------------------------------------------------------------------------------

ODShape* CPCalendar::AdjustBorderShape(Environment* ev, ODFacet* embeddedFacet, ODShape* shape)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_AdjustBorderShape
	EnteringMethod("\pCPCalendar::AdjustBorderShape");
	ODUnused(embeddedFacet);

	// This if for embedding parts
	DebugStr("\pCPCalendar::AdjustBorderShape -- shouldn't be here.");
	shape->Acquire(ev);
	return shape;
}
