//Parts of this file are
//Copyright  1995
//Manic Moose Consulting
//All rights reserved.
//You may use this code for reference purposed only
//moose@manicmoose.com

//----------------------------------------------------------------------------------------
// Imaging protocol
//----------------------------------------------------------------------------------------

#ifndef _CALENDAR_
#include "CPCalendar.h"
#endif

#ifndef __ICONS__
#include <Icons.h>
#endif

#if GENERATING68K
#pragma segment Imaging
#endif

//----------------------------------------------------------------------------------------

void CPCalendar::Draw(Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Draw
	EnteringMethod("\pCPCalendar::Draw");
	ODUnused(invalidShape);

	CFocus	foc(ev, facet);

	ODTypeToken view = facet->GetFrame(ev)->GetViewType(ev);

	if (view == fSession->Tokenize(ev, kODViewAsLargeIcon))
	{
		this->PrivDrawAsLargeIcon(ev, facet, invalidShape);
		return;
	}
	
	if (view == fSession->Tokenize(ev, kODViewAsSmallIcon))
	{
		this->PrivDrawAsSmallIcon(ev, facet, invalidShape);
		return;
	}
	
	if (view == fSession->Tokenize(ev, kODViewAsThumbnail))
	{
		this->PrivDrawAsThumbnail(ev, facet, invalidShape);
		return;
	}
	
	if (view == fSession->Tokenize(ev, kODViewAsFrame))
	{
		ODFrame*	frame     = facet->GetFrame(ev);
		ODShape*	usedShape = frame->AcquireUsedShape(ev, kODNULL);
		RgnHandle	rgn       = usedShape->GetQDRegion(ev);
	
		usedShape->Release(ev);
	
		Rect		rct     = (*rgn)->rgnBBox;
		RgnHandle	oldClip = NULL;
		RgnHandle	newClip;
	
		if (frame->IsRoot(ev))
		{
			GrafPtr	curPort;
			::GetPort(&curPort);
			Rect r = curPort->portRect;
			r.left = r.right  - 15;
			r.top  = r.bottom - 15;
			::GetClip(oldClip = NewRgn());
			::ClipRect(&r);
			::DrawGrowIcon(curPort);
			::GetClip(newClip = ::NewRgn());
			::DiffRgn(oldClip, newClip, newClip);
			::SetClip(newClip);
			::DisposeRgn(newClip);
			::EraseRect(&(curPort->portRect));
		}
	
		// draw a gratuitous string, and capture the image for thumbnail display
		if (fThumbnailPicture)
			::KillPicture(fThumbnailPicture);
		fThumbnailPicture = NULL;
		fThumbnailPicture = ::OpenPicture(&rct);
		PenState	saveState;
		GetPenState(&saveState);
		PenNormal();
		monthDrawer.DrawSelf(rct);
		rct.left--;
		SetPenState(&saveState);
		::ClosePicture();
		::DrawPicture(fThumbnailPicture, &rct);
	
		if (oldClip) {
			::SetClip(oldClip);
			::DisposeRgn(oldClip);
		}
	}
}

//----------------------------------------------------------------------------------------

static pascal OSErr PrivDetachIconResource(ResType theType, Handle *theIcon, void* yourDataPtr)
{
	if (theIcon != kODNULL && *theIcon != kODNULL)
	{
		::DetachResource(*theIcon);
		return ::ResError();
	}
	else
		return noErr;
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivDrawAsLargeIcon(Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	Rect				rect32;
	IconTransformType 	transformType = ttNone;
	Handle				iconSuite;
	
	// Check to see if the icon is selected and in an active window
	if ( facet->GetHighlight(ev) == kODFullHighlight && facet->GetWindow(ev)->IsActive(ev) )
		transformType = ttSelected;
	
	// Draw the icon.
	{
		CUsingLibraryResources res;
		OSErr error = ::GetIconSuite(&iconSuite, kHelloIconSuite, svAllLargeData);		
	
		if ( error == noErr )
		{
			// Create a universal proc pointer for our icon action proc.
			IconActionUPP actionProc = NewIconActionProc(PrivDetachIconResource);
			error = ::ForEachIconDo(iconSuite, svAllLargeData, actionProc, kODNULL);
			DisposeRoutineDescriptor(actionProc);
		}
	}
	
	::SetRect(&rect32, 0, 0, kODLargeIconSize, kODLargeIconSize);
	::PlotIconSuite(&rect32, atAbsoluteCenter, transformType, iconSuite);

	if ( facet->GetHighlight(ev) == kODDimHighlight )
	{
		ODShape* usedShape = facet->GetFrame(ev)->AcquireUsedShape(ev, kODNULL);
		RgnHandle usedRgn = usedShape->GetQDRegion(ev);
		
		::PenPat(&ODQDGlobals.gray);
		::FrameRgn(usedRgn);
		::PenNormal();
		
		usedShape->Release(ev);
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivDrawAsSmallIcon(Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	Rect				rect16;
	IconTransformType 	transformType = ttNone;
	Handle				iconSuite;
	
	// Check to see if the icon is selected and in an active window
	if ( facet->GetHighlight(ev) == kODFullHighlight && facet->GetWindow(ev)->IsActive(ev) )
		transformType = ttSelected;
	
	// Draw the icon.
	{
		CUsingLibraryResources res;
		OSErr error = ::GetIconSuite(&iconSuite, kHelloIconSuite, svAllSmallData);		
	
		if ( error == noErr )
		{
			// Create a universal proc pointer for our icon action proc.
			IconActionUPP actionProc = NewIconActionProc(PrivDetachIconResource);
			error = ::ForEachIconDo(iconSuite, svAllLargeData, actionProc, kODNULL);
			DisposeRoutineDescriptor(actionProc);
		}
	}
	
	::SetRect(&rect16, 0, 0, kODSmallIconSize, kODSmallIconSize);
	::PlotIconSuite(&rect16, atAbsoluteCenter, transformType, iconSuite);

	if ( facet->GetHighlight(ev) == kODDimHighlight )
	{
		ODShape* usedShape = facet->GetFrame(ev)->AcquireUsedShape(ev, kODNULL);
		RgnHandle usedRgn = usedShape->GetQDRegion(ev);
		
		::PenPat(&ODQDGlobals.gray);
		::FrameRgn(usedRgn);
		::PenNormal();
		
		usedShape->Release(ev);
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivDrawAsThumbnail(Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	// The last image displayed as frame is used as a Thumbnail
	Rect rct;
	::SetRect(&rct, 0, 0, kODThumbnailSize, kODThumbnailSize);
	if (fThumbnailPicture)
		::DrawPicture(fThumbnailPicture, &rct);
}
//----------------------------------------------------------------------------------------

void CPCalendar::CanvasUpdated(Environment* ev, ODCanvas* canvas)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_CanvasUpdated
	EnteringMethod("\pCPCalendar::CanvasUpdated");
	ODUnused(canvas);
}

//----------------------------------------------------------------------------------------

void CPCalendar::HighlightChanged(Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_HighlightChanged
	EnteringMethod("\pCPCalendar::HighlightChanged");
	ODUnused(facet);
}

//----------------------------------------------------------------------------------------

ODULong CPCalendar::GetPrintResolution(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_GetPrintResolution
	EnteringMethod("\pCPCalendar::GetPrintResolution");
	ODUnused(frame);
	return 0;
}

//----------------------------------------------------------------------------------------

void CPCalendar::FacetAdded(Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_FacetAdded
	EnteringMethod("\pCPCalendar::FacetAdded");
	
	ODFrame* frame = facet->GetFrame(ev);
	this->PrivAdjustFacets(ev, frame);
		// Given a new negotiated frame shape, adjust our facets based
		// on it.  That's all there is to it.

	if (frame->IsRoot(ev))
	{
		frame->Invalidate(ev, kODNULL, kODNULL);

		ODBoolean succeeded = kODFalse;
		succeeded = fSession->GetArbitrator(ev)->RequestFocusSet(ev, fFocusSet,frame);
		if (succeeded)
		{
			this->FocusAcquired(ev, fSelectionFocus, frame);
			this->FocusAcquired(ev, fMenuFocus, frame);
			this->FocusAcquired(ev, fKeyFocus, frame);
		}
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::FacetRemoved(Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_FacetRemoved
	EnteringMethod("\pCPCalendar::FacetRemoved");
}

//----------------------------------------------------------------------------------------

void CPCalendar::CanvasChanged(Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_CanvasChanged
	EnteringMethod("\pCPCalendar::CanvasChanged");
	ODUnused(facet);
}

//----------------------------------------------------------------------------------------

void CPCalendar::GeometryChanged(Environment* ev, ODFacet* facet,
												   ODBoolean clipShapeChanged,
												   ODBoolean externalTransformChanged)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_GeometryChanged
	EnteringMethod("\pCPCalendar::GeometryChanged");
	ODUnused(facet);
	ODUnused(clipShapeChanged);
	ODUnused(externalTransformChanged);
}