//Parts of this file are
//Copyright  1995
//Manic Moose Consulting
//All rights reserved.
//You may use this code for reference purposed only
//moose@manicmoose.com

//----------------------------------------------------------------------------------------
// UI Events protocol
//----------------------------------------------------------------------------------------

#ifndef _CALENDAR_
#include "CPCalendar.h"
#endif

const ODPropertyName	kODPropSourceLink	= "Calendar:Property:SourceLink";
const ODPropertyName	kODPropDestLink		= "Calendar:Property:DestLink";

#ifndef SOM_ODLinkSource_xh
#include <LinkSrc.xh>
#endif

#ifndef SOM_ODLink_xh
#include <Link.xh>
#endif

#ifndef __DRAG__
#include <Drag.h>
#endif

#ifndef _STDTYPIO_
#include <StdTypIO.h>
#endif

//----------------------------------------------------------------------------------------

void CPCalendar::InitPart(Environment* ev, ODStorageUnit* storageUnit, ODPart* self)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_InitPart
	EnteringMethod("\pCPCalendar::InitPart");

	this->PrivInitCalendar(ev, storageUnit, self);

	{
		CUsingLibraryResources fil;
		::GetIndString(fTextData, kContentStringResID, kContentStringID);
	}		// Use CUsingLibraryResources to use your part's resource fork.
			// Make it go out of scope when you are done.

	// Initialize your other fields here.
	fDirty = kODTrue;
		// Make sure our content is written out at least once.
}

//----------------------------------------------------------------------------------------

void CPCalendar::InitPartFromStorage(Environment* ev, ODStorageUnit* storageUnit, ODPart* self)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_InitPartFromStorage
	EnteringMethod("\pCPCalendar::InitPartFromStorage");

	TRY
			
		// Call the common initialization code to get set up.
		this->PrivInitCalendar(ev, storageUnit, self);
	
		// Read in the state the part was in when it was last Externalized.
		// This allows the part to present the same "environment" the user
		// had the part set up in the last time it was edited.
		this->PrivInternalizeStateInfo(ev, storageUnit);
	
		// Read in the contents for your part editor.
		this->PrivInternalizeContent(ev, storageUnit);

		// Read in the links
		this->PrivInternalizeLinks(ev, storageUnit);

	CATCH_ALL
		ODSetSOMException(ev, ErrorCode());

		// Clean up will occur in the destructor which will be called
		// shortly after we THROW the error.
	ENDTRY
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivInitCalendar(Environment* ev, ODStorageUnit* storageUnit, ODPart* self)
{
	EnteringMethod("\pCPCalendar::PrivInitCalendar");

	// To allow editor swapping (translation) at runtime, OpenDoc requires
	// that we pass in a "reference" to ourselves when interacting with the
	// API (ie. WindowState::RegisterWindow(), Dispatcher::RegisterIdle, etc).
	// The "partWrapper" passed to us here and in InitPart is the
	// "reference" OpenDoc is asking us to use.
	fSelf = self;
	fDisplayFrames = new FrameList;

	// Are we being opened from a read-only draft? If so, we cannot
	// write anything back out to our storage unit.
	fReadOnlyStorage = ( storageUnit->GetDraft(ev)->
									GetPermissions(ev) == kDPReadOnly );
			
	// Save off some references.
	fStorageUnit = storageUnit;
	fDraft = fStorageUnit->GetDraft(ev);
	fSession = fStorageUnit->GetSession(ev);
		// As your part grows, add your initialization that is common
		// between InitPart and InitPartFromStorage here.
		// Note that GetStorageUnit can be used, since just prior to
		// calling this method, InitPart called InitPersistentObject or
		// InitPartFromStorage called InitPersistentObjectFromStorage.

	fSelectionFocus = fSession->Tokenize(ev, kODSelectionFocus);
	fMenuFocus      = fSession->Tokenize(ev, kODMenuFocus);
	fKeyFocus       = fSession->Tokenize(ev, kODKeyFocus);
	fModalFocus     = fSession->Tokenize(ev, kODModalFocus);

	fFocusSet = fSession->GetArbitrator(ev)->CreateFocusSet(ev);
	fFocusSet->Add(ev, fKeyFocus);
	fFocusSet->Add(ev, fMenuFocus);
	fFocusSet->Add(ev, fSelectionFocus);

	fMenuBar = fSession->GetWindowState(ev)->CopyBaseMenuBar(ev);

	{
		CUsingLibraryResources fil;
		fMenu = ::GetMenu(kMenuID);
		if (fMenu)
			::DetachResource((Handle)fMenu);		// Must detach it!
	}

	if (!fMenu)
		DebugStr("\pCPCalendar::PrivInitCalendar -- couldn't create menu.");

	fMenuBar->AddMenuLast(ev, kMenuID, fMenu, fSelf);
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivInternalizeStateInfo(Environment* ev, ODStorageUnit* storageUnit)
{
	// Description:	This method could be used to read in settings
	//				information for the part. This would be information
	//				related to the workings of the part editor, not the
	//				content.
	
	EnteringMethod("\pPrivInternalizeStateInfo");

	ODStorageUnitRef	weakRef;
	ODULong				size;

	// Internalize the part's display frame list.

		unsigned long	theTime;
		GetDateTime(&theTime);
		DateTimeRec		timeRec;
		SecondsToDate(theTime, &timeRec);
		month = timeRec.month;
		year = timeRec.year;

		monthDrawer.Set(month, year, ev);

	if (ODSUExistsThenFocus(ev, storageUnit, kODPropDisplayFrames, kODWeakStorageUnitRefs))
	{									
		size = storageUnit->GetSize(ev);
		storageUnit->SetOffset(ev, 0);	
	
		for (ODULong offset = 0; offset < size; offset += 4)
		{	
			TRY	
				StorageUnitGetValue(storageUnit, ev, sizeof(ODStorageUnitRef), (ODPtr)&weakRef);
				
				if ( storageUnit->IsValidStorageUnitRef(ev, weakRef) )
				{		
					ODFrame *frame = storageUnit->GetDraft(ev)->
						AcquireFrame(ev, storageUnit->GetIDFromStorageUnitRef(ev, weakRef));
					fDisplayFrames->Add(frame, ev);
					frame->Release(ev);
				}
			SOM_ENDTRY
		}
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivInternalizeContent(Environment* ev, ODStorageUnit* storageUnit)
{
	// Description:	This method is called during internalization of the
	//				part. The content of the part should be read in.
	EnteringMethod("\pInternalizeContent");
	
	if (ODSUExistsThenFocus(ev, storageUnit, kODPropContents, kCalendarKind))
	{
		//fTextData[0] = (unsigned char)storageUnit->GetSize(ev);
		//StorageUnitGetValue(storageUnit, ev, fTextData[0], &fTextData[1]);
		monthDrawer.Internalize(storageUnit, ev);
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivInternalizeLinks(Environment* ev, ODStorageUnit *storageUnit)
{
 	ODStorageUnitRef aSURef;

	if ( fStorageUnit->Exists(ev, kODPropSourceLink, kCalendarKind, 0))
	{
		ODBoolean validLink = kODTrue;
		fStorageUnit->Focus(ev, kODPropSourceLink, kODPosUndefined, kCalendarKind, 0, kODPosUndefined);
		TRY
			StorageUnitGetValue(fStorageUnit, ev, sizeof(ODStorageUnitRef),&aSURef);
			if ( fStorageUnit->IsValidStorageUnitRef(ev, aSURef))
			{
				fSrcLink = fDraft->AcquireLinkSource(ev, fStorageUnit->GetIDFromStorageUnitRef(ev, aSURef));
				StorageUnitGetValue(fStorageUnit, ev, sizeof(ODUpdateID), (ODValue) &fSrcLinkChange);

				// Ensure the link source references this part
				fSrcLink->SetSourcePart(ev, fStorageUnit);
			}
			else
			{
				validLink = kODFalse;
			}
		CATCH_ALL
			validLink = kODFalse;
		ENDTRY

		if ( !validLink)
		{
			fStorageUnit->SetOffset(ev, 0);
			fStorageUnit->DeleteValue(ev, fStorageUnit->GetSize(ev));
			fSrcLink = kODNULL;
		}
	}

	if ( fStorageUnit->Exists(ev, kODPropDestLink, kCalendarKind, 0))
	{
		ODBoolean validLink = kODTrue;
		fStorageUnit->Focus(ev, kODPropDestLink, kODPosUndefined, kCalendarKind, 0, kODPosUndefined);
		TRY
			StorageUnitGetValue(fStorageUnit, ev, sizeof(ODStorageUnitRef),&aSURef);
			if ( fStorageUnit->IsValidStorageUnitRef(ev, aSURef))
			{
				fDestLink = fDraft->AcquireLink(ev, fStorageUnit->GetIDFromStorageUnitRef(ev, aSURef), (ODLinkSpec*) kODNULL);
				StorageUnitGetValue(fStorageUnit, ev, sizeof(ODLinkInfo), (ODValue) &fDestLinkInfo);
				ODType kind = (ODType) ODNewPtr((ODULong) fDestLinkInfo.kind , kDefaultHeapID);
				StorageUnitGetValue(fStorageUnit, ev, (ODULong) fDestLinkInfo.kind, (ODValue) kind);
				fDestLinkInfo.kind = kind;
	
				// Now register as a dependent of the link
				// Specify kODUnknownUpdate to force getting the text.
				// By default the destination is automatically updated.
				if (fDestLinkInfo.autoUpdate)
					fDestLink->RegisterDependent(ev, fSelf, kODUnknownUpdate);
				fDraft->SetChangedFromPrev(ev);
			}
			else
			{
				validLink = kODFalse;
			}
		CATCH_ALL
			validLink = kODFalse;
		ENDTRY

		if ( !validLink)
		{
			fStorageUnit->SetOffset(ev, 0);
			fStorageUnit->DeleteValue(ev, fStorageUnit->GetSize(ev));
			fDestLink = kODNULL;
		}
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::Externalize(Environment* ev)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Externalize
	EnteringMethod("\pCPCalendar::Externalize");

	if (fDirty && !fReadOnlyStorage)
	{
		// Get our storage unit.
		ODStorageUnit* storageUnit = fSelf->GetStorageUnit(ev);

		// Verify that the storage unit has the appropriate properties
		// and values to allow us to run. If not, add them.
		this->PrivCheckAndAddProperties(ev, storageUnit);
	
		// Verify that there are no "bogus" values in the Content property.
		this->PrivCleanContentProperty(ev, storageUnit);
	
		// Write out the part's state information.
		this->PrivExternalizeStateInfo(ev, storageUnit, 0, kODNULL);
			
		// Write out the part's content.
		this->PrivExternalizeContent(ev, storageUnit);

		// Write out the part's link information.
		this->PrivExternalizeLinks(ev, storageUnit);

		// Flag our part as no longer being dirty.
		fDirty = kODFalse;
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivCheckAndAddProperties(Environment* ev, ODStorageUnit* storageUnit)
{
	// Description:	This method is called during the creation of
	//				stationery, to prepare a storage unit, and during
	//				externalization, to verify that all the properties we
	//				need are present.
	//
	//				The part adds the default content property, a
	//				preferred editor property (to aid in part binding).

    EnteringMethod("\pPrivCheckAndAddProperties");

	// Create our content property and preferred content property kind.

	// Here is how you would do it by directly calling the OpenDoc APIs:
/*
	if ( !storageUnit->Exists(ev, kODPropContents, kODNULL, 0) )
		storageUnit->AddProperty(ev, kODPropContents);
	if ( !storageUnit->Exists(ev, kODPropContents, kCalendarKind, 0) )
		storageUnit->AddValue(ev, kCalendarKind);
*/
	// Since this kind of storageUnit manipulation is fairly common, we use a utility to do it.
	// You'll see more uses of ODSUForceFocus later.
	ODSUForceFocus(ev, storageUnit, kODPropContents, kCalendarKind);
	
/*
// Since we only have ONE kind, we don't have to set up a PreferredKind property
// However, if you're going to support multiple kinds, you should include this section of code
// in your part editor.
	
	if ( !storageUnit->Exists(ev, kODPropPreferredKind, kODISOStr, 0) )
	{
		ODSUForceFocus(ev, storageUnit, kODPropPreferredKind, kODISOStr);
		
		// Since we are setting up the preferred kind property, we just write
		// out our default "kind" for the editor. We can write out the user
		// chosen kind in the PrivExternalizeStateInfo method.

		StorageUnitSetValue(storageUnit, ev, ODISOStrLength(kCalendarKind), kCalendarKind);
	}
*/
	
	// Add our display frame list.
	ODSUForceFocus(ev, storageUnit, kODPropDisplayFrames, kODWeakStorageUnitRefs);
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivCleanContentProperty(Environment* ev, ODStorageUnit* storageUnit)
{
	// Description:	This method is called during exteralization of the
	//				part. The part should remove any value in the content property
	//				that it cannot "accurately" write to.
    EnteringMethod("\pPrivCleanContentProperty");

	storageUnit->Focus(ev, kODPropContents, kODPosUndefined, kODNULL, 0, kODPosAll);
	
	ODULong numValues = storageUnit->CountValues(ev);
	
	for (ODULong index = numValues; index > 1; index--)
	{
		// Index from 1 to n through the values.
		storageUnit->Focus(ev, kODPropContents, kODPosUndefined, 
								kODNULL, index, kODPosUndefined);
		ODValueType value = storageUnit->GetType(ev);
		
		// If the value type is not one we support, remove it.
		if ( ODISOStrCompare(value, kCalendarKind) != 0 )
			storageUnit->Remove(ev);
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivExternalizeStateInfo(Environment*		ev,
									   ODStorageUnit*	storageUnit,
									   ODDraftKey		key,
									   ODFrame*			scopeFrame)
{
	// Description:	This method is called during externalization of the
	//				part. The current "state" of the editor should be
	//				written out. This "state" information may be lost
	//				during Data Interchange operations, so the part needs
	//				to recover gracefully if information is missing or
	//				incomplete.
	
    EnteringMethod("\pPrivExternalizeStateInfo");

	ODStorageUnitRef	weakRef;
	ODFrame*			frame;
	ODID				frameID;
	ODID				scopeFrameID = ( scopeFrame ? scopeFrame->GetID(ev) : 0 );
	ODDraft*			fromDraft = fSelf->GetStorageUnit(ev)->GetDraft(ev);
	
	// Externalize the part's display frame list.
	if (ODSUExistsThenFocus(ev, storageUnit, kODPropDisplayFrames, kODWeakStorageUnitRefs))
	{
		// Persistent object references are stored in a side table, rather than
		// in the property/value stream. Thus, deleting the contents of a value
		// will not "delete" the references previously written to that value. To
		// completely "delete" all references written to the value, we must
		// remove the value,
		storageUnit->Remove(ev);
	}
		// and add it back.
	ODSUForceFocus(ev, storageUnit, kODPropDisplayFrames, kODWeakStorageUnitRefs);
	
	for (FrameLink* link = fDisplayFrames->First(); link->Frame(); link = link->Next() )
	{
		frame = (ODFrame*) link->Frame();
		frameID = frame->GetID(ev);
		
		// If a draft key exists, then we are being cloned to another draft.
		// We must "weak" clone our display frame and reference the cloned
		// frame. The part re-uses the frameID variable so there aren't two
		// different GetWeakStorageUnitRef calls.
		if ( key )
			frameID = fromDraft->WeakClone(ev, key, frameID, 0, scopeFrameID);
		
		// Write out weak references to each of the part's display frames.
		storageUnit->GetWeakStorageUnitRef(ev, frameID, weakRef);
		TRY
			StorageUnitSetValue(storageUnit, ev, sizeof(ODStorageUnitRef), (ODPtr)&weakRef);
		SOM_ENDTRY
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivExternalizeContent(Environment* ev, ODStorageUnit* storageUnit)
{
	// Description:	This method is called during externalization of the
	//				part. The content of the part should be written out.
    EnteringMethod("\pPrivExternalizeContent");
	/*
	ODSUForceFocus(ev, storageUnit, kODPropContents, kCalendarKind);
	ODULong	oldSize = storageUnit->GetSize(ev);
		// Focus on content property, get size of the old data in there.
		
	StorageUnitSetValue(storageUnit, ev, fTextData[0], &fTextData[1]);
		// Now we write out the property.
		
	ODULong newSize = storageUnit->GetOffset(ev);
	if (newSize < oldSize)
		storageUnit->DeleteValue(ev, oldSize - newSize);
	*/
	monthDrawer.Externalize(storageUnit, ev);
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivExternalizeLinks(Environment* ev, ODStorageUnit* storageUnit)
{
	ODStorageUnitRef	aSURef;

	if ( fSrcLink == kODNULL)
	{
		if ( fStorageUnit->Exists(ev, kODPropSourceLink, kCalendarKind, 0) )
		{
			fStorageUnit->Focus(ev, kODPropSourceLink, kODPosUndefined, kCalendarKind, (ODValueIndex)0, kODPosUndefined);
			fStorageUnit->Remove(ev);
		}
	}
	else
	{
		fSrcLink->Externalize(ev);
		if ( fStorageUnit->Exists(ev, kODPropSourceLink, kCalendarKind, 0) )
			fStorageUnit->Focus(ev, kODPropSourceLink, kODPosUndefined, kCalendarKind, (ODValueIndex)0, kODPosUndefined);
		else
			fStorageUnit->AddProperty(ev,kODPropSourceLink)->AddValue(ev,kCalendarKind);
		fStorageUnit->GetStrongStorageUnitRef(ev, fSrcLink->GetStorageUnit(ev)->GetID(ev), aSURef);
		StorageUnitSetValue( fStorageUnit, ev, sizeof(ODStorageUnitRef), aSURef);
	}

	if ( fDestLink == kODNULL)
	{
		if ( fStorageUnit->Exists(ev, kODPropDestLink, kCalendarKind, 0) )
		{
			fStorageUnit->Focus(ev, kODPropDestLink, kODPosUndefined, kCalendarKind, (ODValueIndex)0, kODPosUndefined);
			fStorageUnit->Remove(ev);
		}
	}
	else 
	{
		fDestLink->Externalize(ev);
		if ( fStorageUnit->Exists(ev, kODPropDestLink, kCalendarKind, 0) )
			fStorageUnit->Focus(ev, kODPropDestLink, kODPosUndefined, kCalendarKind, (ODValueIndex)0, kODPosUndefined);
		else
			fStorageUnit->AddProperty(ev,kODPropDestLink)->AddValue(ev,kCalendarKind);
		fStorageUnit->GetStrongStorageUnitRef(ev, fDestLink->GetStorageUnit(ev)->GetID(ev), aSURef);
		StorageUnitSetValue(fStorageUnit, ev, sizeof(ODStorageUnitRef), aSURef);

		// store dest link info kind as ISOStr
		ODType saveKind = fDestLinkInfo.kind;
		fDestLinkInfo.kind = (ODType) ODISOStrLength(saveKind) + 1;
		StorageUnitSetValue(fStorageUnit, ev, sizeof(ODLinkInfo), (ODValue) &fDestLinkInfo );
		StorageUnitSetValue(fStorageUnit, ev, (ODULong)fDestLinkInfo.kind, (ODValue) saveKind );
		fDestLinkInfo.kind = saveKind;
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivSetDirty(Environment* ev)
{
	fDirty = kODTrue;
	fStorageUnit->GetDraft(ev)->SetChangedFromPrev(ev);
	this->PrivUpdateSourceLink(ev);
	this->PrivInvalAllDisplayFrames(ev);
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivPrepareSU(Environment* ev, ODStorageUnit *storageUnit)
{
	ODSUForceFocus(ev, storageUnit, kODPropContents, kCalendarKind);
//	if ( !storageUnit->Exists(ev, kODPropContents, kODNULL, 0) )
//		storageUnit->AddProperty(ev, kODPropContents);
//	if ( !storageUnit->Exists(ev, kODPropContents, kCalendarKind, 0) )
//		storageUnit->AddValue(ev, kCalendarKind);
}

//----------------------------------------------------------------------------------------

ODBoolean CPCalendar::PrivInternalizeFromSU(Environment* ev, ODStorageUnit *su, ODFrame *frame)
{
	Boolean gotSomething = kODFalse;
	
	// make new TextEdit record
	Rect theRect;
	::SetRect(&theRect, 0, 0, 0, 0);
	TEHandle tempTE			= TEStyleNew(&theRect, &theRect);
	StScrpHandle stylHandle = kODNULL;
	OSErr error				= noErr;
	short fileRefNum		= 0;
	Size textSize			= 0;
	ODPtr textData			= kODNULL;
	ODPtr fileData			= kODNULL;

	TRY
	
	// Try our native format first
	if (ODSUExistsThenFocus(ev, su, kODPropContents, kCalendarKind))
	{
		textSize = su->GetSize(ev);
		textData = ODNewPtr(textSize, kDefaultHeapID);
		StorageUnitGetValue(su, ev, textSize, (ODValue)textData);
		
		gotSomething = kODTrue;
	}
	// Then try standard Apple Roman TEXT format
	else if (ODSUExistsThenFocus(ev, su, kODPropContents, kODAppleTEXT))
	{
		textSize = su->GetSize(ev);
		textData = ODNewPtr(textSize, kDefaultHeapID);
		StorageUnitGetValue(su, ev, textSize, (ODValue)textData);
	
		if (ODSUExistsThenFocus(ev, su, kODPropContents, kODApplestyl))
		{
			Size stylSize = su->GetSize(ev);
			stylHandle = (StScrpHandle) ODNewHandle(stylSize);
			StorageUnitGetValue(su, ev, stylSize, (ODValue) ODLockHandle((ODHandle) stylHandle));
			ODUnlockHandle((ODHandle) stylHandle);
		}
		
		gotSomething = kODTrue;
		
	} 
	// And finally check to see if someone dropped a file, and just slurp in its entire data fork.
	else if (ODSUExistsThenFocus(ev, su, kODPropContents, kODApplehfs))
	{
		long fileSize = su->GetSize(ev);		// really hfsflavor size
		fileData = ODNewPtr(fileSize, kDefaultHeapID);			// really hfsflavor data
		StorageUnitGetValue(su, ev, fileSize, (ODValue)fileData);
		
		THROW_IF_ERROR(::FSpOpenDF((FSSpec*)&(((HFSFlavor*)fileData)->fileSpec), fsRdPerm, &fileRefNum));
		THROW_IF_ERROR(::GetEOF(fileRefNum, &textSize));
		if (textSize == 0)
			THROW(kODErrUndefined);
			
		textData = ODNewPtr(textSize, kDefaultHeapID);
		
		THROW_IF_ERROR(::SetFPos(fileRefNum,fsFromStart,0));
		THROW_IF_ERROR(::FSRead(fileRefNum,&textSize,textData));
		THROW_IF_ERROR(::FSClose(fileRefNum));

		gotSomething = kODTrue;
	}
	
	TESetSelect(32767, 32767, tempTE);
	TEStyleInsert(textData, textSize, stylHandle, tempTE);
	
	if (textData)
		ODDisposePtr(textData);
	if (stylHandle)
		ODDisposeHandle((ODHandle) stylHandle);

	CATCH_ALL
	if (tempTE != kODNULL)
		::TEDispose(tempTE);
	if (fileData != kODNULL)
		ODDisposePtr(fileData);
	if (stylHandle != kODNULL)
		ODDisposeHandle((ODHandle) stylHandle);
	if (fileRefNum != 0)
		::FSClose(fileRefNum);
	gotSomething = kODFalse;
	ENDTRY

	// OK, we got the styled text and we're now moving that into a Str255
	if (gotSomething)
	{
		Size textSize = (**tempTE).teLength;
		ODPtr textData = ODNewPtr(textSize, kDefaultHeapID);
		if (! textData) 
		{
			DebugStr("\pNewPtr error");
			::TEDispose(tempTE);
			return kODDropFail ;
		}
		
		// In reality, you would preserve the style information
		if (textSize)
		{
			Size maxSize = 255 - sizeof("\p by PartMaker");
			textSize = (textSize > maxSize) ? maxSize : textSize;
			ODBlockMove(*(**tempTE).hText, &fTextData[1], textSize);
			fTextData[0] = textSize;
		}
		::TEDispose(tempTE);
	}
		
	return gotSomething;
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivExternalizeToSU(Environment* ev, ODStorageUnit *storageUnit, ODFrame *frame)
{
	this->PrivExternalizeContent(ev, storageUnit);
}
