//----------------------------------------------------------------------------------------
// From Linking protocol
//----------------------------------------------------------------------------------------

#ifndef _CALENDAR_
#include "CPCalendar.h"
#endif

#ifndef SOM_ODBaseLink_xh
#include <LinkB.xh>
#endif

#ifndef SOM_ODBaseLinkSource_xh
#include <LinkSrcB.xh>
#endif

#ifndef SOM_ODLinkSource_xh
#include <LinkSrc.xh>
#endif

#ifndef SOM_ODLinkSpec_xh
#include <LinkSpec.xh>
#endif

#ifndef SOM_ODLink_xh
#include <Link.xh>
#endif

#ifndef _BARRAY_
#include <BArray.h>
#endif

//----------------------------------------------------------------------------------------

ODLinkSource* CPCalendar::CreateLink(Environment* ev, ODByteArray* data)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_CreateLink
	EnteringMethod("\pCPCalendar::CreateLink");

	// Call a part-specific method to get the link if it already exists
	if (fSrcLink == kODNULL)
	{
		// Call this part's draft to create both the link source object
		// used by this part, and the link object returned from this
		// method and used by destinations of the link.

		fSrcLink = fSelf->GetStorageUnit(ev)->GetDraft(ev)->CreateLinkSource(ev, fSelf);
		
		ODLinkKey key;
		ODVolatile(key);
		TRY
			if (fSrcLink->Lock(ev, 0, &key))
			{
			TRY
				ODStorageUnit* linkContentSU = fSrcLink->GetContentStorageUnit(ev, key);
				this->PrivPrepareSU(ev, linkContentSU);
				this->PrivExternalizeToSU(ev, linkContentSU);
				
				// A new change ID is used to update the link the first time.
				ODUpdateID change = fSession->UniqueUpdateID(ev);
				fSrcLink->ContentUpdated(ev, change, key);
				fSrcLinkChange = change;
				fSrcLink->Unlock(ev, key);
			CATCH_ALL
				fSrcLink->Unlock(ev, key);
				RERAISE;
			ENDTRY
			}
		CATCH(kODErrBrokenLinkSource)
			DebugStr("\pPictPart: ODLinkSource::Lock() returned broken link error\n");
			this->PrivRemoveSourceLink(ev);
		ENDTRY
	}

	// update all my facets
	this->PrivSetDirty(ev);
	this->PrivInvalAllDisplayFrames(ev);
	
	return fSrcLink;
}

//----------------------------------------------------------------------------------------

void CPCalendar::LinkUpdated(Environment* ev, ODLink* updatedLink, ODUpdateID id)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_LinkUpdated
	EnteringMethod("\pCPCalendar::LinkUpdated");
	ODUnused(id);

	ODLinkKey key;
	if (updatedLink->Lock(ev, 0, &key))
	{
		ODBoolean gotSomething = kODFalse;
		
		ODStorageUnit* su = updatedLink->GetContentStorageUnit(ev, key);
		if (this->PrivValidDataOnSU(ev, su))
		{
			gotSomething = this->PrivInternalizeFromSU(ev, su);
			fDestLinkInfo.change = fDestLink->GetUpdateID(ev);
			fDestLinkInfo.changeTime = fDestLink->GetChangeTime(ev);
			this->PrivSetDirty(ev);
		}
		updatedLink->Unlock(ev, key);
		if (gotSomething)	
			this->PrivInvalAllDisplayFrames(ev);
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::RevealLink(Environment* ev, ODLinkSource* linkSource)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_RevealLink
	EnteringMethod("\pCPCalendar::RevealLink");

	if (linkSource != fSrcLink)
		return;

	ODFrame* frame = fDisplayFrames->First()->Frame();
	
	if (frame)
	{
		ProcessSerialNumber psn;
		::GetCurrentProcess(&psn);
		::SetFrontProcess(&psn);

		ODFrame* containingFrame = frame->AcquireContainingFrame(ev);

		if (containingFrame)
		{
			ODPart* part = containingFrame->AcquirePart(ev);
			ODVolatile(part);
			// Sample parts currently THROW so catch and ignore
			TRY
				part->RevealFrame(ev, frame, kODNULL);
			CATCH_ALL
			ENDTRY
			part->Release(ev);
		}
		containingFrame->Release(ev);

		// Try to grab all of our foci
		if (fSession->GetArbitrator(ev)->RequestFocusSet(ev, fFocusSet,frame))
		{
			this->FocusAcquired(ev, fSelectionFocus, frame);
			this->FocusAcquired(ev, fMenuFocus, frame);
			this->FocusAcquired(ev, fKeyFocus, frame);
			
			/*
			CFocus myFocus(ev, facet);
			PictPartInfoRec* thisFramePartInfo = (PictPartInfoRec*)frame->GetPartInfo(ev);

			::InvertRect(&thisFramePartInfo->activeRect);
			long finalTicks;
			::Delay(10, &finalTicks);
			::InvertRect(&thisFramePartInfo->activeRect);
			*/
		}
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::LinkStatusChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_LinkStatusChanged
	EnteringMethod("\pCPCalendar::LinkStatusChanged");
	ODUnused(frame);
}

//----------------------------------------------------------------------------------------

void CPCalendar::EmbeddedFrameUpdated(Environment* ev, ODFrame* frame, ODUpdateID change)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_EmbeddedFrameUpdated
	EnteringMethod("\pCPCalendar::EmbeddedFrameUpdated");
	ODUnused(change);
	ODUnused(frame);
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivUpdateSourceLink(Environment* ev)
{
	if (fSrcLink)
	{
		ODLinkKey linkKey;
		ODLinkSource* linkSource = fSrcLink;
		ODUpdateID changeID = fSrcLinkChange;
		
		ODVolatile(linkSource);
		ODVolatile(linkKey);
	
		TRY
		if (linkSource->Lock(ev, 0, &linkKey))
		{
		TRY
			fSrcLinkChange = fSession->UniqueUpdateID(ev);
			changeID = fSrcLinkChange;

			// Remove the current link from the link
			// linkSource->Clear(ev, fSession->UniqueUpdateID(ev), linkKey);
							
			ODStorageUnit* linkContentSU = linkSource->GetContentStorageUnit(ev, linkKey);
					
			// Begin a transaction if objects are cloned into the link
			ODDraftKey draftKey = 0;
			ODVolatile(fDraft);
			ODVolatile(draftKey);	
			ODFrame* frame = fDisplayFrames->First()->Frame(); // $$$$$ T: Won't work for > 1 display frame per part!
			TRY
				draftKey = fDraft->BeginClone(ev,
										linkContentSU->GetDraft(ev),
										frame,
										kODCloneToLink);
				this->PrivPrepareSU(ev, linkContentSU);
				this->PrivExternalizeToSU(ev, linkContentSU);
	
				fDraft->EndClone(ev, draftKey);
	
			CATCH_ALL
	
				if (draftKey != 0)
					fDraft->AbortClone(ev, draftKey);
				RERAISE;
	
			ENDTRY
	
			// Inform the link that it has changed, so destinations can
			// update. The changeID parameter should be the change
			// causing the link to update.
	
			linkSource->ContentUpdated(ev, changeID, linkKey);
			linkSource->Unlock(ev, linkKey);
	
		CATCH_ALL
	
			// Clear a partially-updated link
			linkSource->Clear(ev, fSession->UniqueUpdateID(ev), linkKey);
			linkSource->Unlock(ev, linkKey);
	
		ENDTRY
		}
		CATCH(kODErrBrokenLinkSource)
			DebugStr("\pPictPart: ODLinkSource::Lock() returned broken link error\n");
			this->PrivRemoveSourceLink(ev);
		ENDTRY
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivRemoveSourceLink(Environment* ev)
{
	if (fSrcLink)
	{
		fSrcLink->SetSourcePart(ev, kODNULL);
		fSrcLink->Release(ev);
		fSrcLink = kODNULL;
		fSrcLinkChange = kODUnknownUpdate;
		
		this->PrivSetDirty(ev);
		this->PrivInvalAllDisplayFrames(ev);
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivRemoveDestLink(Environment* ev)
{
	if (fDestLink)
	{
		if (fDestLinkInfo.autoUpdate)
			fDestLink->UnregisterDependent(ev, fSelf);
		fDestLink->Release(ev);
		fDestLink = kODNULL;
	
		this->PrivSetDirty(ev);
		this->PrivInvalAllDisplayFrames(ev);
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivGetPartInfo(Environment* ev, ODFrame* frame, ODEventData* event)
{
	if (fShowLinkInfo)
	{
		ODLinkInfoResult infoResult;
		ODFrameFacetIterator*	facets = frame->CreateFacetIterator(ev);
		if (fSrcLink)
		{
			if (fSrcLink->ShowLinkSourceInfo(ev, 
							facets->First(ev), fSrcLinkChange, kODFalse, &infoResult))
			{
				switch (infoResult.action)
				{
				case kODLinkInfoBreakLink:
					this->PrivRemoveSourceLink(ev);
					break;
				case kODLinkInfoUpdateNow:
					this->PrivUpdateSourceLink(ev);
					break;
				case kODLinkInfoOk :
					if (infoResult.autoUpdate != fSrcLink->IsAutoUpdate(ev))
					{
						fSrcLink->SetAutoUpdate(ev, infoResult.autoUpdate);
						if (infoResult.autoUpdate && fSrcLinkChange != fSrcLink->GetUpdateID(ev))
						{
							// Changed from manual to automatic updates, and the link
							// is out of date; update it now
							this->PrivUpdateSourceLink(ev);
						}
					}
					break;
				}
			}
		}
		else
			if (fDestLink)
			{
				if (fDestLink->ShowLinkDestinationInfo(ev, 
												facets->First(ev),
												&fDestLinkInfo,
												kODFalse, 
												&infoResult))
				{
					switch (infoResult.action)
					{
					case kODLinkInfoFindSource:
						TRY
							fDestLink->ShowSourceContent(ev);
						CATCH_ALL
							DebugStr("\pDragText : Error returned by ODLink::ShowSourceContent");
						ENDTRY;
						break;
					case kODLinkInfoBreakLink:
						this->PrivRemoveDestLink(ev);
						break;
					case kODLinkInfoUpdateNow:
						this->LinkUpdated(ev, fDestLink, fDestLink->GetUpdateID(ev));
						break;
					case kODLinkInfoOk:
						if (infoResult.autoUpdate != fDestLinkInfo.autoUpdate)
						{
							fDestLinkInfo.autoUpdate = infoResult.autoUpdate;
							if (infoResult.autoUpdate)
								fDestLink->RegisterDependent(ev, fSelf, fDestLinkInfo.change);
							else
								fDestLink->UnregisterDependent(ev, fSelf);
						}
						break;
					default:
						break;
					}
				}
			}
		delete facets;
	} else {
		ODFrameFacetIterator* facets = frame->CreateFacetIterator(ev);
		fSession->GetInfo(ev)->ShowPartFrameInfo(ev, facets->First(ev), kODTrue);
		delete facets;
	}
}

//----------------------------------------------------------------------------------------

const ODISOStr kDummyLinkSpecData = "LinkSpec dummy data";

void CPCalendar::PrivExternalizeLinkToSU(Environment* ev, ODStorageUnit *storageUnit)
{
	// Add a link spec to the storage unit if we don't already have a source link
	// (or the source link is selected exactly),
	// and as long as the selection does not include a link destination.
	// Also, don't add a link spec if draft permissions don't allow writing.
	
	ODBoolean makeLinkSpec = kODFalse;
	
	if ( fDraft->GetPermissions(ev) >= kDPSharedWrite)
	{
		if (fSrcLink == kODNULL)
		{
			if (fDestLink == kODNULL)
			{
				makeLinkSpec = kODTrue;
			}
		}		
		else
			makeLinkSpec = kODTrue;
	}
	
	if (makeLinkSpec)
	{
		storageUnit->AddProperty(ev, kODPropLinkSpec);
		ODByteArray data = CreateByteArrayStruct(kDummyLinkSpecData, ODISOStrLength(kDummyLinkSpecData));		
		ODLinkSpec* linkSpec = fDraft->CreateLinkSpec(ev, fSelf, &data);
		linkSpec->WriteLinkSpec(ev, storageUnit);
		DisposeByteArrayStruct(data);
		delete linkSpec;
	}
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivInternalizeLinkFromSU(Environment* ev, ODStorageUnit *storageUnit, 
								ODPasteAsResult *pasteAsResult)
{
	if (storageUnit->Exists(ev, kODPropLinkSpec, (ODValueType) kODNULL, 0)) 
	{
		ODLinkSpec* linkSpec = fDraft->CreateLinkSpec(ev, fSelf, 0);
		storageUnit->Focus(ev, kODPropLinkSpec, kODPosUndefined, (ODValueType)kODNULL, 0, kODPosUndefined);
		linkSpec->ReadLinkSpec(ev, storageUnit);
		TRY
		{
		fDestLink = fDraft->AcquireLink(ev, (ODStorageUnitID) kODNULL, linkSpec);
		if (fDestLink != kODNULL)
		{
			fDestLinkInfo.change = fDestLink->GetUpdateID(ev);
			GetDateTime((ODTime*) &fDestLinkInfo.creationTime);
			fDestLinkInfo.changeTime = fDestLinkInfo.creationTime;
			fDestLinkInfo.kind = pasteAsResult->selectedKind;
			fDestLinkInfo.autoUpdate = pasteAsResult->autoUpdateSetting;

			// Now register as a dependent of the link
			// Specify kODUnknownUpdate to force getting the text.
			// By default the destination is automatically updated.
			if (fDestLinkInfo.autoUpdate)
				fDestLink->RegisterDependent(ev, fSelf, kODUnknownUpdate);
			fDraft->SetChangedFromPrev(ev);
		}
		else
			SysBeep( 30);
		}
		CATCH(kODErrDocNotSaved)
			DebugStr("\pSource document not saved!");
		CATCH_ALL
		ENDTRY
		delete linkSpec;
	}
}

//----------------------------------------------------------------------------------------

ODBoolean CPCalendar::PrivValidLinkOnSU(Environment* ev, ODStorageUnit *storageUnit)
{
	return storageUnit->Exists(ev, kODPropLinkSpec, (ODValueType) kODNULL, 0);
}

