/*
	File:		FrameList.cpp
	Contains:	Linked list of frames, for use by parts
	Written by:	PartMaker

	Change History (most recent first):

	<3>	4/6/95	T		Changed from FacetList to FrameList
	<2>	3/8/95	JS		Updated to b1c13/c14
	<1>	2/2/95	RA		first checked in
	<0>	PartMaker source by Eric Soldan, Tantek elik, Jens Alfke, Reggie Adkins
*/

#ifndef _FRAMELIST_
#include "FrameList.h"
#endif

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifdef applec
#pragma segment FrameList
#endif


//========================================================================================
// FrameLink
//========================================================================================

FrameLink::FrameLink( )
{
	fFrame = kODNULL;
	fPrev  = this;
	fNext  = this;
}

//----------------------------------------------------------------------------------------

FrameLink::FrameLink( ODFrame *frame, FrameLink *list )
{
	fFrame = frame;
	fPrev  = list;
	fNext  = list->fNext;
	list->fNext  = this;
	fNext->fPrev = this;
}

//----------------------------------------------------------------------------------------

FrameLink::~FrameLink( )
{
	if ( fPrev )
		fPrev->fNext = fNext;
	if ( fNext )
		fNext->fPrev = fPrev;
}


//========================================================================================
// FrameLink
//========================================================================================

FrameList::FrameList()
{
	fFrame = kODNULL;
	fPrev = kODNULL;
	fNext = this;
}

//----------------------------------------------------------------------------------------

FrameList::~FrameList( )
{
	Environment*	ev = somGetGlobalEnvironment();

	// Delete all links:
	FrameLink* link 	= kODNULL;
	FrameLink* nextLink	= kODNULL; // Need this so we don't call ->Next() on a deleted link.
	
	for ( link = this->First();
		 (link != this) && link->Frame(); 
		 link = nextLink ) 
	{
		nextLink = link->Next();
		link->Frame()->Release(ev);	// T: fDisplayFrames was hanging onto frame
		delete link;
	}
	if ((link == this) && link->Frame())
		link->Frame()->Release(ev);
}

//----------------------------------------------------------------------------------------

void FrameList::Add( ODFrame *frame, Environment* ev )
{
	// see if the frame already exists, do nothing if it does
	for (FrameLink* link = this->First(); link->Frame(); link = link->Next() ) {
		if ( link->Frame() == frame ) {
			return;
		}
	}

	frame->Acquire(ev); // TC: fDisplayFrames hangs onto frame
	// add new frame, constructor does all the work
	new FrameLink(frame,this);
}

//----------------------------------------------------------------------------------------

void FrameList::Remove( ODFrame *frame, Environment* ev )
{
	for (FrameLink* link = this->First(); link->Frame(); link = link->Next() ) {
		if ( link->Frame() == frame ) {
			frame->Release(ev);	// T: fDisplayFrames was hanging onto frame
			delete link;
			return;
		}
	}
}


