/*
	File:		CPCalendar.cpp
	Contains:	CPCalendar OpenDoc part editor class implementation.
				Built using PartMaker, then modified.
	Written by:	PartMaker

	Change History (most recent first):

	<3>	4/14/95	JS		Updated for Developer Release 2
	<4>	3/8/95	JS		Updated to b1c13/c14
	<3>	3/8/95	RA		Add tokenized kODPresDefault param to RegisterWindow().
	<2>	2/21/95	RA		Add kODFrameObject param to RegisterWindow().
	<1>	2/2/95	RA		first checked in
	<0>	PartMaker source by T. elik, E. Soldan, J. Alfke, R. Adkins, D. Nelson, J. Schalk
*/

//Parts of this file are
//Copyright  1995
//Manic Moose Consulting
//All rights reserved.
//You may use this code for reference purposed only
//moose@manicmoose.com

#ifndef _CALENDAR_
#include "CPCalendar.h"
#endif

#ifndef _ODUTILS_
#include "ODUtils.h"
#endif

#ifndef SOM_CalendarSI_xh
#include "CalendarSI.xh"
#endif

//----------------------------------------------------------------------------------------

CPCalendar::CPCalendar():
	monthDrawer(*this)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Constructor
	EnteringMethod("\pCPCalendar::CPCalendar");

	fDirty    			= kODFalse;
	fWindowID			= 0;
	fSession  			= kODNULL;
	fSelf 	  			= kODNULL;
	fStorageUnit		= kODNULL;
	fMenuBar  			= kODNULL;
	fDraftKey 			= 0;
	fThumbnailPicture	= kODNULL;
	
	fSrcLink 			= kODNULL;
	fSrcLinkChange		= kODUnknownUpdate;
	fDestLink			= kODNULL;
	fDraft				= kODNULL;
	fTextData[0]		= 0;
	fSemanticInterface	= kODNULL;
	fDisplayFrames		= kODNULL;
}

//----------------------------------------------------------------------------------------

CPCalendar::~CPCalendar()
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Destructor
	EnteringMethod("\pCPCalendar::~CPCalendar");

	if (fThumbnailPicture)
		::KillPicture(fThumbnailPicture);
	fThumbnailPicture = NULL;
}

//----------------------------------------------------------------------------------------

void CPCalendar::Release(Environment* ev)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Release
	EnteringMethod("\pCPCalendar::Release");

	if (fSelf->GetRefCount(ev) == 1) {
		// Here is where you would unregister idle time for the part if
		// have registered time.  Note that if you unregister without
		// registering, OpenDoc will crash.  It's not just a simple
		// look-up-and-if-there-remove kind of operation.

		// Note that if you unregister idle here, OpenDoc will need to
		// call Release (here) due to it.  This means that when we call
		// to unregister the idle, we are nesting calls to Release.
		// Due to this, I return out of each case, just to make the
		// flow of control of the call(s) to Release more straightforward.

		// Note that if you are going to use idle registration, you will
		// need to include "Disptch.h"

		return;
	}
}

//----------------------------------------------------------------------------------------

ODID CPCalendar::Open(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Open
	EnteringMethod("\pCPCalendar::Open");

	ODWindow* window = kODNULL;

	if (frame) // Doing a View In Window or Open Root
	{
		if (frame->IsRoot(ev))
		{
			fWindowID = this->PrivOpenSavedWindow(ev, frame);
		}
		else
		{ 
			// View In Window
			// First check to see if it is already 
			window = fSession->GetWindowState(ev)->AcquireWindow(ev, fWindowID);
			if (window)
			{
				window->Select(ev);
			}
			else
			{
				window = this->PrivCreateWindow(ev, frame);
				fWindowID = window->GetID(ev);
				window->Open(ev);
				window->Show(ev);
				window->Select(ev);			
			}
			ODReleaseObject(ev, window);	
		}
	}
	else
	{
		fWindowID = this->PrivOpenInitialWindow(ev);
	}
	return fWindowID;
}

//----------------------------------------------------------------------------------------

ODWindow*  CPCalendar::PrivCreateWindow(Environment *ev,ODFrame* sourceFrame)
{
	// get rectangle
	Rect windRect;
	const ODSShort		kOnePageWidth = 600;
	::SetRect(&windRect, 4, GetMBarHeight() + 24,
			ODQDGlobals.screenBits.bounds.right - 64,
			ODQDGlobals.screenBits.bounds.bottom - 4);
	if (windRect.right - windRect.left > kOnePageWidth)
		windRect.right = windRect.left + kOnePageWidth;

	// get title
	Str255 windowTitleStr;
	ODName*	partName = ODGetPOName(ev, fSelf, kODNULL);
	IntlToPStr(partName, (StringPtr)&windowTitleStr);
	DisposeIText(partName);

	ODPlatformWindow platformWindow = ::NewCWindow(kODNULL,
													&windRect,
													windowTitleStr,
													false,
													this->PrivWantResizableWindow() ? zoomDocProc : zoomNoGrow,
													(WindowPtr)-1L,
													true,
													kODNULL);

	ODBoolean isRoot = ((sourceFrame == kODNULL) || sourceFrame->IsRoot(ev));

	ODWindow* window = fSession->GetWindowState(ev)->RegisterWindow(ev,
							platformWindow,								// newWindow
							isRoot || (sourceFrame->GetStorageUnit(ev)) ? 	// same as sourceFrame
								kODFrameObject : 						// 	frameType
								kODNonPersistentFrameObject,			
							isRoot,										// isRootWindow
							this->PrivWantResizableWindow(),					// isResizable
							kODFalse,									// isFloating
							kODTrue,									// shouldSave
							fSelf,										// rootPart
							fSession->Tokenize(ev, kODViewAsFrame),		// viewType
							fSession->Tokenize(ev, kODPresDefault),		// presentation
							sourceFrame);								// sourceFrame
	return window;
}

//----------------------------------------------------------------------------------------

ODID CPCalendar::PrivOpenInitialWindow(Environment* ev)
{
	ODWindow* window = this->PrivCreateWindow(ev, kODNULL);
	ODID windowID = window->GetID(ev);
	window->Open(ev);
	window->Show(ev);
	window->Select(ev);
	window->Release(ev);
	return windowID;			
}

//----------------------------------------------------------------------------------------

ODID CPCalendar::PrivOpenSavedWindow(Environment* ev, ODFrame* frame)
{
	ODID windowID = 0;
	ODWindow* window = kODNULL;
	WindowProperties props;
	
	if (BeginGetWindowProperties(ev, frame, &props))
	{
		ODPlatformWindow platformWindow = NewCWindow(kODNULL, &(props.boundsRect), props.title, 
					kODFalse, props.procID, (WindowPtr)-1L, props.hasCloseBox, props.refCon);
		
		window = fSession->GetWindowState(ev)->RegisterWindowForFrame(ev, 
										platformWindow, 
										frame,
										props.isRootWindow,	// Keeps draft open
										kODTrue,	// Is resizable
										kODFalse,	// Is floating
										kODTrue,	// shouldSave
										props.sourceFrame);
		EndGetWindowProperties(ev, &props); // Release source frame
				
		window->Open(ev);
	}
	else
	{
		Rect windRect;
		SetRect(&windRect, 100, 100, 300, 300);	
		ODPlatformWindow platformWindow = NewCWindow(kODNULL, &windRect, "\p", kODFalse, zoomDocProc, (WindowPtr)-1L, true, kODNULL);
				
		window =  fSession->GetWindowState(ev)->RegisterWindowForFrame(ev, platformWindow, 
														frame,
														kODTrue,	// Keeps draft open
														kODTrue,	// Is resizable
														kODFalse,	// Is floating
														kODTrue,	// should save
														kODNULL);
		window->Open(ev);	
		window->AdjustWindowShape(ev);
	}
	window->Show(ev);

	windowID = window->GetID(ev);
	window->Release(ev);
	
	return windowID;
}

//----------------------------------------------------------------------------------------

ODBoolean CPCalendar::PrivWantResizableWindow()
{
	return kODFalse;
}

//----------------------------------------------------------------------------------------

void CPCalendar::ReleaseAll(Environment* ev)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_ReleaseAll
	EnteringMethod("\pCPCalendar::ReleaseAll");

	if (fMenuBar != kODNULL)
		fMenuBar->Release(ev);
	ODReleaseObject(ev, fSemanticInterface);
	ODDeleteObject(fDisplayFrames);
}

//----------------------------------------------------------------------------------------

void CPCalendar::PrivInvalAllDisplayFrames(Environment* ev)
{
	// This is just a CPCalendar utility method.

	for (FrameLink *fl = fDisplayFrames->First(); fl->Frame(); fl = fl->Next())
		fl->Frame()->Invalidate(ev, kODNULL, kODNULL);
}

//----------------------------------------------------------------------------------------
// Storage protocol
//----------------------------------------------------------------------------------------

void CPCalendar::Acquire(Environment* ev)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Acquire
	EnteringMethod("\pCPCalendar::Acquire");
}

//----------------------------------------------------------------------------------------

void CPCalendar::CloneInto(Environment* ev, ODDraftKey key, 
		ODStorageUnit* destinationSU, ODFrame* initiatingFrame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_CloneInto
	EnteringMethod("\pCPCalendar::CloneInto");

	// We must first verify that we've never written to this storage unit.
	// If we have, we should do nothing, otherwise we need to write out
	// the current state of the part content.
	
	if ( destinationSU->Exists(ev, kODPropContents, kCalendarKind, 0) == kODFalse )
	{
		// Add the properties we need to successfully externalize
		// ourselves into the destination storage unit.
		this->PrivCheckAndAddProperties(ev, destinationSU);
				
		// Write out the part's state information.
		this->PrivExternalizeStateInfo(ev, destinationSU, key, initiatingFrame);
			
		// Write out the part's content.
		this->PrivExternalizeContent(ev, destinationSU);
	}
}

//----------------------------------------------------------------------------------------

ODSize CPCalendar::Purge(Environment* ev, ODSize size)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Purge
	ODUnused(size);
	EnteringMethod("\pCPCalendar::Purge");

	// CPCalendar doesn't do anything here.
	return 0;
}
