//Copyright  1995
//Manic Moose Consulting
//All rights reserved.
//You may use this code for reference purposed only
//moose@manicmoose.com

#include "Day.h"
#include <TextEdit.h>
#include <TextUtils.h>
#include <Memory.h>
#ifndef _STDTYPIO_
#include <StdTypIO.h>
#endif
#include <StdProps.h>
#ifndef _STORUTIL_
#include <StorUtil.h>
#endif

Day::Day():data(0)
{
}

Day::~Day()
{
	if (data)
		DisposePtr((Ptr)data);
}

void	Day::Draw(const Rect &frame)
{
	Str255	buffer;
	Rect	frm = frame;
	
	if (day > 0)
		{
			NumToString(day, buffer);
			frm.right--;
			frm.bottom--;
			frm.top = frm.bottom-lineHeight;
			frm.left = frm.right-lineHeight*2;
			if ((frm.left >= frame.left) && (frm.top >= frame.top))
				{
					TETextBox(buffer+1, buffer[0], &frm, teJustRight);
				}
			if (data)
				{
					//DebugStr("\pdrawing");
					frm = frame;
					frm.top += lineHeight;
					if ((frm.right - frm.left >= lineHeight*5) && (frm.bottom - frm.top >= lineHeight))
						TETextBox(data+1, data[0], &frm, teJustLeft);
				}
		}
	FrameRect(&frame);
}

void convertToLabel(int year, int month, int day, ODValueType value);
void convertToLabel(int year, int month, int day, char *string)
{
	BlockMove("ManicMoose:Kind:Calendar:", string, 25);
	string[25] = year/1000 + '0';
	string[26] = year/100 - (year/1000)*10 + '0';
	string[27] = year/10 - (year/100)*10 + '0';
	string[28] = year - (year/10)*10 + '0';
	string[29] = month/10 + '0';
	string[30] = month - (month/10)*10 + '0';
	string[31] = day/10 + '0';
	string[32] = day - (day/10)*10 + '0';
	string[33] = 0;
}

void	Day::Externalize(int year, int month, ODStorageUnit *su, Environment *ev)
{
	char	string[255];
	
	if (data)
		{
			convertToLabel(year, month, day, string);
			ODSUForceFocus(ev, su, kODPropContents, string);
			
			ODULong	oldSize = su->GetSize(ev);
				// Focus on content property, get size of the old data in there.
				
			StorageUnitSetValue(su, ev, data[0], &data[1]);
				// Now we write out the property.
				
			ODULong newSize = su->GetOffset(ev);
			if (newSize < oldSize)
				su->DeleteValue(ev, oldSize - newSize);
		}
	
}

void	Day::Internalize(int year, int month, ODStorageUnit *su, Environment *ev)
{
	char	string[255];
	//DebugStr("\pinternalize");
	if (day > 0)
		{
			convertToLabel(year, month, day, string);
			if (ODSUExistsThenFocus(ev, su, kODPropContents, string))
				{
					data = (unsigned char *)NewPtr(su->GetSize(ev)+1);
					data[0] = (unsigned char)su->GetSize(ev);
					StorageUnitGetValue(su, ev, data[0], &data[1]);
					//DebugStr("\pinternalize");
				}
		}
}

void	Day::SetData(const char *newData, UInt8 length)
{
	if (data)
		DisposePtr((Ptr)data);
	data = (unsigned char *)NewPtr(length +1);
	data[0] = (unsigned char)length;
	BlockMove(newData, data+1, length);
}
