//Copyright  1995
//Manic Moose Consulting
//All rights reserved.
//You may use this code for reference purposed only
//moose@manicmoose.com

#include "Month.h"
#include "Week.h"
#include "CPCalendar.h"

static	Str15	DayOfWeekNames[] = {"\pNoDay", "\pSunday", "\pMonday", "\pTuesday", "\pWednsday", "\pThursday", "\pFriday", "\pSaturday"};
static	Str15	MonthNames[] = {"\pNoMonth", "\pJanuary", "\pFebruary", "\pMarch", "\pApril", "\pMay", "\pJune", "\pJuly", "\pAugust", "\pSeptember", "\pOctober", "\pNovember", "\pDecember"};
static	Uint8	LastDay[] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};


Uint8	Month::GetFirstOfMonth(Uint8 newMonth, Uint16 newYear)
{
	DateTimeRec	firstOfMonth;
	Uint32		secondsOfFirstOfMonth;
	
	firstOfMonth.day = 1;
	firstOfMonth.month = newMonth;
	firstOfMonth.year = newYear;
	firstOfMonth.hour = 0;
	firstOfMonth.minute = 0;
	firstOfMonth.second = 1;
	DateToSeconds(&firstOfMonth, &secondsOfFirstOfMonth);
	SecondsToDate(secondsOfFirstOfMonth, &firstOfMonth);
	return firstOfMonth.dayOfWeek;
}


void	Month::Set(Uint8 newMonth, Uint16 newYear, Environment *ev)
{
	month = newMonth;
	year = newYear;
	
	Int8	firstDay = GetFirstOfMonth(newMonth, newYear);
	Int8	start = 2-firstDay;
	numberOfWeeks = (LastDay[newMonth] + firstDay + 5)/7;
	weeks[0].Set(start, LastDay[newMonth]);
	start += 7;
	weeks[1].Set(start, LastDay[newMonth]);
	start += 7;
	weeks[2].Set(start, LastDay[newMonth]);
	start += 7;
	weeks[3].Set(start, LastDay[newMonth]);
	if (numberOfWeeks > 4)
		{
			start += 7;
			weeks[4].Set(start, LastDay[newMonth]);
		}
	if (numberOfWeeks > 5)
		{
			start += 7;
			weeks[5].Set(start, LastDay[newMonth]);
		}
	Internalize(calendar.StorageUnit(), ev);
}

void	Month::DrawSelf(const Rect& frame)
{
	Rect	weekFrame;
	Uint8	height, count;
	Int16	fontNum;
	UInt8	slop;
	
	GetFNum("\pGeneva", &fontNum);
	TextFont(fontNum);
	TextSize(9);
	height = (frame.bottom - frame.top)/numberOfWeeks;
	slop = (frame.bottom - frame.top)%numberOfWeeks;
	weekFrame = frame;
	weekFrame.left++;
	weekFrame.bottom = weekFrame.top+height;
	if (slop)
		{
			weekFrame.bottom++;
			slop--;
		}
	for (count = 0; count < numberOfWeeks; count++)
		{
			weeks[count].Draw(weekFrame);
			weekFrame.top = weekFrame.bottom-1;
			weekFrame.bottom += height;
			if (slop)
				{
					weekFrame.bottom++;
					slop--;
				}
		}
}

Day	*Month::GetDayInPoint(const Rect& frame, Point pt)
{
	Rect	weekFrame;
	Uint8	height, count;
	UInt8	slop;
		
	height = (frame.bottom - frame.top)/numberOfWeeks;
	slop = (frame.bottom - frame.top)%numberOfWeeks;
	weekFrame = frame;
	weekFrame.left++;
	weekFrame.bottom = weekFrame.top+height;
	if (slop)
		{
			weekFrame.bottom++;
			slop--;
		}
	
	for (count = 0; count < numberOfWeeks; count++)
		{
			if (PtInRect(pt, &weekFrame))
				return weeks[count].GetDayInPoint(weekFrame, pt);
			weekFrame.top = weekFrame.bottom-1;
			weekFrame.bottom += height;
			if (slop)
				{
					weekFrame.bottom++;
					slop--;
				}
		}
	return 0;
}

Boolean	Month::CalcDayFrameFromPoint(const Rect& frame, Point pt, Rect &returnRect)
{
	Rect	weekFrame;
	Uint8	height, count;
	UInt8	slop;
		
	height = (frame.bottom - frame.top)/numberOfWeeks;
	slop = (frame.bottom - frame.top)%numberOfWeeks;
	weekFrame = frame;
	weekFrame.left++;
	weekFrame.bottom = weekFrame.top+height;
	if (slop)
		{
			weekFrame.bottom++;
			slop--;
		}
	
	for (count = 0; count < numberOfWeeks; count++)
		{
			if (PtInRect(pt, &weekFrame))
				return weeks[count].CalcDayFrameFromPoint(weekFrame, pt, returnRect);
			weekFrame.top = weekFrame.bottom-1;
			weekFrame.bottom += height;
			if (slop)
				{
					weekFrame.bottom++;
					slop--;
				}
		}
	return 0;
}

void	Month::Externalize(ODStorageUnit *su, Environment *ev)
{
	for (UInt8 count = 0; count < numberOfWeeks; count++)
		weeks[count].Externalize(year, month, su, ev);
}	

void	Month::Internalize(ODStorageUnit *su, Environment *ev)
{
	for (UInt8 count = 0; count < numberOfWeeks; count++)
		weeks[count].Internalize(year, month, su, ev);
}	
