//Copyright  1995
//Manic Moose Consulting
//All rights reserved.
//You may use this code for reference purposed only
//moose@manicmoose.com

#include "Week.h"
#include "Day.h"

void	Week::Set(Int8 startingDay, Int8 lastDate)
{
	Uint8	count;
	for (count = 0; count < 7; count++)
		if (startingDay <= lastDate)
			days[count] = (startingDay++);
		else
			days[count] = 0;
}


void	Week::Draw(const Rect &frame)
{
	Uint8	count, slop = (frame.right - frame.left)%7;
	Rect	dayFrame;
	Uint16	width = (frame.right - frame.left)/7;
	
	dayFrame = frame;
	dayFrame.right = dayFrame.left+width;
	if (slop)
		{
			dayFrame.right++;
			slop--;
		}
	for (count = 0; count < 7; count++)
		{
			days[count].Draw(dayFrame);
			dayFrame.left = dayFrame.right-1;
			dayFrame.right += width;
			if (slop)
				{
					dayFrame.right++;
					slop--;
				}
		}
}

Day		*Week::GetDay(UInt8 day)
{
	UInt8	count;
	for (count = 0; count < 7; count++)
		if (days[count] == day)
			return &days[count];
	return 0;
}

void	Week::Externalize(int year, int month, ODStorageUnit *su, Environment *ev)
{
	Uint8	count;
	for (count = 0; count < 7; count++)
		days[count].Externalize(year, month, su, ev);	
}

void	Week::Internalize(int year, int month, ODStorageUnit *su, Environment *ev)
{
	Uint8	count;
	for (count = 0; count < 7; count++)
		days[count].Internalize(year, month, su, ev);	
}

Day	*Week::GetDayInPoint(const Rect& frame, Point pt)
{
	Uint8	count, slop = (frame.right - frame.left)%7;
	Rect	dayFrame;
	Uint16	width = (frame.right - frame.left)/7;
	
	dayFrame = frame;
	dayFrame.right = dayFrame.left+width;
	if (slop)
		{
			dayFrame.right++;
			slop--;
		}
	for (count = 0; count < 7; count++)
		{
			if (PtInRect(pt, &dayFrame))
				return &days[count];
			dayFrame.left = dayFrame.right-1;
			dayFrame.right += width;
			if (slop)
				{
					dayFrame.right++;
					slop--;
				}
		}
	return 0;
}

Boolean	Week::CalcDayFrameFromPoint(const Rect& frame, Point pt, Rect &returnRect)
{
	Uint8	count, slop = (frame.right - frame.left)%7;
	Rect	dayFrame;
	Uint16	width = (frame.right - frame.left)/7;
	
	dayFrame = frame;
	dayFrame.right = dayFrame.left+width;
	if (slop)
		{
			dayFrame.right++;
			slop--;
		}
	for (count = 0; count < 7; count++)
		{
			if (PtInRect(pt, &dayFrame))
				{
					returnRect = dayFrame;
					return true;
				}
			dayFrame.left = dayFrame.right-1;
			dayFrame.right += width;
			if (slop)
				{
					dayFrame.right++;
					slop--;
				}
		}
	return false;
}
